import os
import sys
import shutil
from dash.unzip_data_files import unzip_data_files


def download_file(filename, urlpath, printStatus, scriptDirectory):
    dataFilename = os.path.join(scriptDirectory, filename)
    if (not os.path.isfile(dataFilename)) and (not os.path.isdir(dataFilename.strip(".zip"))):
        print(printStatus)
        if sys.version_info[0] < 3:
            import urllib
            dataFileDownload = urllib.URLopener()
            dataFileDownload.retrieve(urlpath, dataFilename)
        else:
            import urllib.request
            urllib.request.urlretrieve(urlpath, dataFilename)

        print("Download complete! File saved to %s." % dataFilename)
        if filename.endswith(".zip"):
            unzip_data_files(filename)


def delete_previous_versions(oldFilenames, scriptDirectory):
    for oldFilename in oldFilenames:
        dataFilename = os.path.join(scriptDirectory, oldFilename)
        if os.path.isfile(dataFilename):
            print("Deleting previous version of data file: %s" % oldFilename)
            os.remove(dataFilename)


def download_all_files(zipVersion):
    scriptDirectory = os.path.dirname(os.path.abspath(__file__))

    oldFilenames = ['model_trainedAtZeroZ.ckpt', 'type_age_atRedshiftZero.npz', 'training_params.pickle', 'templates.npz',
                    'model_trainedAtZeroZ_v02.ckpt', 'type_age_atRedshiftZero_v02.npz', 'training_params_v02.pickle', 'templates_v02.npz',
                    'model_trainedAtZeroZ_v03.ckpt', 'type_age_atRedshiftZero_v03.npz', 'training_params.pickle', 'templates_v03.npz',
                    'model_trainedAtZeroZ_v03.ckpt.data-00000-of-00001', 'model_trainedAtZeroZ_v03.ckpt.index', 'model_trainedAtZeroZ_v03.ckpt.meta',]
    delete_previous_versions(oldFilenames, scriptDirectory)

    saveFilenames = ['models_{0}.zip'.format(zipVersion)]

    urlpaths = ["https://github.com/daniel-muthukrishna/DASH/blob/master/dash/models_{0}.zip?raw=true".format(zipVersion)]

    printStatuses = ["Downloading data files... \nThis download contains the Tensorflow models trained using deep learning. \n"
                     "The file to be downloaded is ~200MB. However, this is a one time download, and will only occur\n"
                     "the very first time that this version of DASH is installed."]

    for i in range(len(urlpaths)):
        download_file(saveFilenames[i], urlpaths[i], printStatuses[i], scriptDirectory)


if __name__ == '__main__':
    download_all_files('v01')






import numpy as np
import tensorflow as tf
from dash.multilayer_convnet import convnet_variables


def calc_model_statistics(modelFilename, testImages, testTypeNames, typeNamesList):
    nw = len(testImages[0])
    nBins = len(typeNamesList)
    imWidthReduc = 8
    imWidth = 32  # Image size and width

    x, y_, keep_prob, y_conv = convnet_variables(imWidth, imWidthReduc, nw, nBins)

    saver = tf.train.Saver()

    with tf.Session() as sess:
        saver.restore(sess, modelFilename)

        yy = y_conv.eval(feed_dict={x: testImages, keep_prob: 1.0})

    # ACTUAL ACCURACY, broadTYPE ACCURACY, AGE ACCURACY
    typeAndAgeCorrect = 0
    typeCorrect = 0
    broadTypeCorrect = 0
    broadTypeAndAgeCorrect = 0
    typeAndNearAgeCorrect = 0
    broadTypeAndNearAgeCorrect = 0
    for i in range(len(testTypeNames)):
        predictedIndex = np.argmax(yy[i])

        classification = testTypeNames[i].split(': ')
        if len(classification) == 2:
            testType, testAge = classification
        else:
            testGalType, testType, testAge = classification
        actual = typeNamesList[predictedIndex].split(': ')
        if len(actual) == 2:
            actualType, actualAge = actual
        else:
            actualGalType, actualType, actualAge = actual

        testBroadType = testType[0:2]
        actualBroadType = actualType[0:2]
        if testType[0:3] == 'IIb':
            testBroadType = 'Ib'
        if actualType[0:3] == 'IIb':
            actualBroadType = 'Ib'
        nearTestAge = testAge.split(' to ')

        if testTypeNames[i] == typeNamesList[predictedIndex]:
            typeAndAgeCorrect += 1
        if testType == actualType:  # correct type
            typeCorrect += 1
            if (nearTestAge[0] in actualAge) or (nearTestAge[1] in actualAge):  # check if the age is in the neigbouring bin
                typeAndNearAgeCorrect += 1  # all correct except nearby bin
        if testBroadType == actualBroadType:  # correct broadtype
            broadTypeCorrect += 1
            if testAge == actualAge:
                broadTypeAndAgeCorrect += 1
            if (nearTestAge[0] in actualAge) or (nearTestAge[1] in actualAge):  # check if the age is in the neigbouring bin
                broadTypeAndNearAgeCorrect += 1  # Broadtype and nearby bin

    typeAndAgeAccuracy = float(typeAndAgeCorrect) / len(testTypeNames)
    typeAccuracy = float(typeCorrect) / len(testTypeNames)
    broadTypeAccuracy = float(broadTypeCorrect) / len(testTypeNames)
    broadTypeAndAgeAccuracy = float(broadTypeAndAgeCorrect) / len(testTypeNames)
    typeAndNearAgeAccuracy = float(typeAndNearAgeCorrect) / len(testTypeNames)
    broadTypeAndNearAgeAccuracy = float(broadTypeAndNearAgeCorrect) / len(testTypeNames)

    print("typeAndAgeAccuracy : " + str(typeAndAgeAccuracy))
    print("typeAccuracy : " + str(typeAccuracy))
    print("broadTypeAccuracy : " + str(broadTypeAccuracy))
    print("broadTypeAndAgeAccuracy: " + str(broadTypeAndAgeAccuracy))
    print("typeAndNearAgeAccuracy : " + str(typeAndNearAgeAccuracy))
    print("broadTypeAndNearAgeAccuracy : " + str(broadTypeAndNearAgeAccuracy))


if __name__ == '__main__':
    dirModel = "/Users/danmuth/PycharmProjects/DASH/dash/models/agnosticZ/data_files_agnosticZ_noHost/"
    modelFilename = dirModel + "tensorflow_model.ckpt"

    dirTestSet = "/Users/danmuth/PycharmProjects/DASH/dash/models/agnosticZ/data_files_agnosticZ_noHost/training_set/"
    testImages = np.load(dirTestSet + 'testImages.npy', mmap_mode='r')
    # testLabelsIndexes = np.load(dir1 + 'testLabels.npy', mmap_mode='r')
    typeNamesList = np.load(dirTestSet + 'typeNamesList.npy')
    testTypeNames = np.load(dirTestSet + 'testTypeNames.npy')

    calc_model_statistics(modelFilename, testImages[:1000], testTypeNames[:1000], typeNamesList)



import os
import sys
import pickle
import numpy as np
from dash.download_data_files import download_all_files
from dash.restore_model import LoadInputSpectra, BestTypesListSingleRedshift, get_training_parameters, classification_split
from dash.false_positive_rejection import FalsePositiveRejection, combined_prob
from dash.read_binned_templates import load_templates, get_templates, ReadBinnedTemplates
from dash.calculate_redshift import get_median_redshift

try:
    from PyQt5 import QtGui
    from dash.gui_main import MainApp
except ImportError:
    print("Warning: You will need to install 'PyQt5' if you want to use the graphical interface. "
          "Using the automatic library will continue to work.")


class Classify(object):
    def __init__(self, filenames=[], redshifts=[], smooth=6, minWave=2500, maxWave=10000, classifyHost=False, knownZ=True, data_files='models_v01'):
        """ Takes a list of filenames and corresponding redshifts for supernovae.
        Files should contain a single spectrum, and redshifts should be a list of corresponding redshift floats
        """
        # download_all_files('v01')
        self.filenames = filenames
        self.redshifts = redshifts
        self.smooth = smooth
        self.minWave = minWave
        self.maxWave = maxWave
        self.classifyHost = classifyHost
        self.numSpectra = len(filenames)
        self.scriptDirectory = os.path.dirname(os.path.abspath(__file__))
        if knownZ and redshifts != []:
            self.knownZ = True
        else:
            self.knownZ = False
        self.pars = get_training_parameters()
        self.nw, w0, w1 = self.pars['nw'], self.pars['w0'], self.pars['w1']
        self.dwlog = np.log(w1/w0)/self.nw
        self.snTemplates, self.galTemplates = load_templates(os.path.join(self.scriptDirectory, data_files, 'models/sn_and_host_templates.npz'))

        if self.knownZ:
            if classifyHost:
                self.modelFilename = os.path.join(self.scriptDirectory, data_files, "models/zeroZ_classifyHost/tensorflow_model.ckpt")
            else:
                self.modelFilename = os.path.join(self.scriptDirectory, data_files, "models/zeroZ/tensorflow_model.ckpt")
        else:
            if self.classifyHost:
                self.modelFilename = os.path.join(self.scriptDirectory, data_files, "models/agnosticZ_classifyHost/tensorflow_model.ckpt")
            else:
                self.modelFilename = os.path.join(self.scriptDirectory, data_files, "models/agnosticZ/tensorflow_model.ckpt")

    def _get_images(self, filename, redshift):
        loadInputSpectra = LoadInputSpectra(filename, redshift, redshift, self.smooth, self.pars, self.minWave, self.maxWave, self.classifyHost)
        inputImage, inputRedshift, typeNamesList, nw, nBins = loadInputSpectra.input_spectra()

        return inputImage, typeNamesList, nw, nBins

    def _input_spectra_info(self):
        inputImages = np.empty((0, int(self.nw)), np.float16)
        for i in range(self.numSpectra):
            f = self.filenames[i]
            if self.knownZ:
                z = self.redshifts[i]
            else:
                z = 0
            inputImage, typeNamesList, nw, nBins = self._get_images(f, z)
            inputImages = np.append(inputImages, inputImage, axis=0)
        bestTypesList = BestTypesListSingleRedshift(self.modelFilename, inputImages, typeNamesList, self.nw, nBins)
        bestTypes = bestTypesList.bestTypes
        softmaxes = bestTypesList.softmaxOrdered
        bestLabels = bestTypesList.idx

        return bestTypes, softmaxes, bestLabels, inputImages

    def list_best_matches(self, n=5):
        """Returns a list of lists of the the top n best matches for each spectrum"""
        bestTypes, softmaxes, bestLabels, inputImages = self._input_spectra_info()
        bestMatchLists = []
        bestBroadTypes = []
        rejectionLabels = []
        reliableFlags = []
        redshifts = []
        for specNum in range(self.numSpectra):
            bestMatchList = []
            for i in range(20):
                host, name, age = classification_split(bestTypes[specNum][i])
                if not self.knownZ:
                    redshifts.append(self.calc_redshift(inputImages[i], name, age)[0])
                prob = softmaxes[specNum][i]
                bestMatchList.append((host, name, age, prob))
            bestMatchList = np.array(bestMatchList)
            bestMatchLists.append(bestMatchList[0:n])
            bestBroadType, reliableFlag = self.best_broad_type(bestMatchList)
            bestBroadTypes.append(bestBroadType)
            reliableFlags.append(reliableFlag)
            rejectionLabels.append(self.false_positive_rejection(bestTypes[specNum][0], inputImages[specNum]))

        bestMatchLists = np.array(bestMatchLists)

        if not redshifts:
            redshifts = self.redshifts
        else:
            redshifts = np.array(redshifts)

        return bestMatchLists, redshifts, bestBroadTypes, rejectionLabels, reliableFlags

    def best_broad_type(self, bestMatchList):
        host, prevName, bestAge, probTotal, reliableFlag = combined_prob(bestMatchList[0:10])

        return (prevName, bestAge, probTotal), reliableFlag

    def false_positive_rejection(self, bestType, inputImage):
        host, name, age = classification_split(bestType)
        snInfos, snNames, hostInfos, hostNames = get_templates(name, age, host, self.snTemplates, self.galTemplates, self.nw)
        if snInfos != []:
            templateImages = snInfos[:, 1]
            falsePositiveRejection = FalsePositiveRejection(inputImage, templateImages)
            rejectionLabel = "NONE"  # "(chi2=%s, rlap=%s)" % (falsePositiveRejection.rejection_label(), falsePositiveRejection.rejection_label2())
        else:
            rejectionLabel = "(NO_TEMPLATES)"

        # import matplotlib
        # matplotlib.use('TkAgg')
        # import matplotlib.pyplot as plt
        # plt.plot(inputImage)
        # plt.plot(templateImage)
        # plt.show()

        return rejectionLabel

    def calc_redshift(self, inputFlux, snName, snAge):
        host = "No Host"
        snInfos, snNames, hostInfos, hostNames = get_templates(snName, snAge, host, self.snTemplates, self.galTemplates, self.nw)
        numOfSubTemplates = len(snNames)
        templateFluxes = []
        for i in range(numOfSubTemplates):
            templateFluxes.append(snInfos[i][1])

        redshift, crossCorr = get_median_redshift(inputFlux, templateFluxes, self.nw, self.dwlog)
        print(redshift)
        if redshift is None:
            return 0, np.zeros(1024)

        return round(redshift, 4), crossCorr

    # def save_best_matches(self, n=1, filename='DASH_matches.txt'):
    #     if ('self.bestMatchLists' not in locals()) or self.n != n:
    #         self.bestMatchLists = self.list_best_matches(n)
    #     np.savetxt(filename, self.bestMatchLists)

    def plot_with_gui(self, indexToPlot=0):
        app = QtGui.QApplication(sys.argv)
        form = MainApp(inputFilename=self.filenames[indexToPlot])
        form.lblInputFilename.setText(self.filenames[indexToPlot].split('/')[-1])
        form.checkBoxKnownZ.setChecked(self.knownZ)
        form.checkBoxClassifyHost.setChecked(self.classifyHost)
        form.lineEditKnownZ.setText(str(self.redshifts[indexToPlot]))
        form.lineEditSmooth.setText(str(self.smooth))
        form.classifyHost = self.classifyHost
        form.select_tensorflow_model()
        form.fit_spectra()
        form.show()
        app.exec_()




# # EXAMPLE USAGE:
# classification = Classify(filenames=['/Users/dmuthukrishna/Users/dmuthukrishna/DES16E1dic_E1_combined_161125_v10_b00.dat',
#                                      '/Users/dmuthukrishna/Users/dmuthukrishna/DES16E1dic_E1_combined_161125_v10_b00.dat'],
#                           redshifts=[0.34, 0.13])
# print(classification.list_best_matches())







import numpy as np
from random import shuffle
from dash.sn_processing import PreProcessing
from dash.combine_sn_and_host import CombineSnAndHost
from dash.preprocessing import ProcessingTools
from dash.array_tools import zero_non_overlap_part, normalise_spectrum
import multiprocessing as mp
import random
import time


class AgeBinning(object):
    def __init__(self, minAge, maxAge, ageBinSize):
        self.minAge = minAge
        self.maxAge = maxAge
        self.ageBinSize = ageBinSize

    def age_bin(self, age):
        ageBin = int(round(age / self.ageBinSize)) - int(round(self.minAge / self.ageBinSize))

        return ageBin

    def age_labels(self):
        ageLabels = []

        ageBinPrev = 0
        ageLabelMin = self.minAge
        for age in np.arange(self.minAge, self.maxAge, 0.5):
            ageBin = self.age_bin(age)

            if ageBin != ageBinPrev:
                ageLabelMax = int(round(age))
                ageLabels.append(str(int(ageLabelMin)) + " to " + str(ageLabelMax))
                ageLabelMin = ageLabelMax

            ageBinPrev = ageBin

        ageLabels.append(str(int(ageLabelMin)) + " to " + str(int(self.maxAge)))

        return ageLabels


class CreateLabels(object):

    def __init__(self, nTypes, minAge, maxAge, ageBinSize, typeList, hostList, nHostTypes):
        self.nTypes = nTypes
        self.minAge = minAge
        self.maxAge = maxAge
        self.ageBinSize = ageBinSize
        self.typeList = typeList
        self.ageBinning = AgeBinning(self.minAge, self.maxAge, self.ageBinSize)
        self.numOfAgeBins = self.ageBinning.age_bin(self.maxAge-0.1) + 1
        self.nLabels = self.nTypes * self.numOfAgeBins
        self.ageLabels = self.ageBinning.age_labels()
        self.hostList = hostList
        self.nHostTypes = nHostTypes

    def label_array(self, ttype, age, host=None):
        ageBin = self.ageBinning.age_bin(age)

        try:
            typeIndex = self.typeList.index(ttype)
        except ValueError as err:
            print("INVALID TYPE: {0}".format(err))
            raise ValueError

        if host is None:
            labelArray = np.zeros((self.nTypes, self.numOfAgeBins))
            labelArray[typeIndex][ageBin] = 1
            labelArray = labelArray.flatten()
            typeName = ttype + ": " + self.ageLabels[ageBin]
        else:
            hostIndex = self.hostList.index(host)
            labelArray = np.zeros((self.nHostTypes, self.nTypes, self.numOfAgeBins))
            labelArray[hostIndex][typeIndex][ageBin] = 1
            labelArray = labelArray.flatten()
            typeName = "{}: {}: {}".format(host, ttype, self.ageLabels[ageBin])

        labelIndex = np.argmax(labelArray)

        return labelIndex, typeName

    def type_names_list(self):
        typeNamesList = []
        if self.hostList is None:
            for tType in self.typeList:
                for ageLabel in self.ageBinning.age_labels():
                    typeNamesList.append("{}: {}".format(tType, ageLabel))
        else:
            for host in self.hostList:
                for tType in self.typeList:
                    for ageLabel in self.ageBinning.age_labels():
                        typeNamesList.append("{}: {}: {}".format(host, tType, ageLabel))

        return typeNamesList
        

class TempList(object):
    def temp_list(self, tempFileList):
        f = open(tempFileList, 'rU')

        fileList = f.readlines()
        for i in range(0,len(fileList)):
            fileList[i] = fileList[i].strip('\n')

        f.close()

        return fileList
    

class ReadSpectra(object):

    def __init__(self, w0, w1, nw, snFilename, galFilename=None):
        self.w0 = w0
        self.w1 = w1
        self.nw = nw
        self.snFilename = snFilename
        if galFilename is None:
            self.data = PreProcessing(snFilename, w0, w1, nw)
        else:
            self.galFilename = galFilename
            self.dataCombined = CombineSnAndHost(snFilename, galFilename, w0, w1, nw)

    def sn_plus_gal_template(self, snAgeIdx, snCoeff, galCoeff, z):
        wave, flux, minIndex, maxIndex, nCols, ages, tType = self.dataCombined.training_template_data(snAgeIdx, snCoeff, galCoeff, z)

        return wave, flux, nCols, ages, tType, minIndex, maxIndex

    def snid_template_data(self, ageIdx, z):
        """ lnw template files """
        wave, flux, nCols, ages, tType, minIndex, maxIndex = self.data.snid_template_data(ageIdx, z)

        return wave, flux, nCols, ages, tType, minIndex, maxIndex

    def sf_age(self):
        snName, extension = self.snFilename.strip('.dat').split('.')
        ttype, snName = snName.split('/')

        if extension == 'max':
            age = 0
        elif extension[0] == 'p':
            age = float(extension[1:])
        elif extension[0] == 'm':
            age = -float(extension[1:])
        else:
            print("Invalid Superfit Filename: " + self.snFilename)

        return snName, ttype, age

    def superfit_template_data(self, z):
        """ Returns wavelength and flux after all preprocessing """
        wave, flux, minIndex, maxIndex = self.data.two_column_data(z, minWave=self.w0, maxWave=self.nw)
        snName, ttype, age = self.sf_age()

        print(snName, ttype, age)

        return wave, flux, minIndex, maxIndex, age, snName, ttype

    def input_spectrum(self, z, smooth, minWave, maxWave):
        wave, flux, minIndex, maxIndex = self.data.two_column_data(z, smooth, minWave, maxWave)

        return wave, flux, int(minIndex), int(maxIndex)


class ArrayTools(object):

    def __init__(self, nLabels, nw):
        self.nLabels = nLabels
        self.nw = nw

    def shuffle_arrays(self, images, labels, filenames, typeNames):
        arraySize = len(labels)
        imagesShuf = np.empty((arraySize, int(self.nw)), np.float16)
        labelsShuf = np.empty(arraySize, np.uint16)
        filenamesShuf = np.empty(arraySize, dtype=object)
        typeNamesShuf = np.empty(arraySize, dtype=object)
        idx = 0
        print("Shuffle2")
        # Randomise order
        indexShuf = list(range(arraySize))
        shuffle(indexShuf)
        for i in indexShuf:
            imagesShuf[idx] = images[i]
            labelsShuf[idx] = labels[i]
            filenamesShuf[idx] = filenames[i]
            typeNamesShuf[idx] = typeNames[i]
            idx += 1
        print("LenLabels")
        print(len(labels), idx)
            
        print(imagesShuf)
        print(typeNamesShuf)
        print("Shuffle3")    
        return imagesShuf, labelsShuf, filenamesShuf, typeNamesShuf

    def count_labels(self, labels):
        counts = np.zeros(self.nLabels)

        for i in range(len(labels)):
            counts[labels[i]] += 1

        return counts

    def div0(self, a, b):
        """ ignore / 0, div0( [-1, 0, 1], 0 ) -> [0, 0, 0] """
        with np.errstate(divide='ignore', invalid='ignore'):
            c = np.true_divide(a, b)
            c[~ np.isfinite(c)] = 0  # -inf inf NaN
        return c

    def augment_data(self, flux, stdDevMean=0.05, stdDevStdDev=0.05):
        minIndex, maxIndex = ProcessingTools().min_max_index(flux)
        noise = np.zeros(self.nw)
        stdDev = abs(np.random.normal(stdDevMean, stdDevStdDev)) # randomised standard deviation
        noise[minIndex:maxIndex] = np.random.normal(0, stdDev, maxIndex - minIndex)
        augmentedFlux = flux + noise
        augmentedFlux = normalise_spectrum(augmentedFlux)
        augmentedFlux = zero_non_overlap_part(augmentedFlux, minIndex, maxIndex)

        return augmentedFlux

    def over_sample_arrays(self, images, labels, filenames, typeNames):
        counts = self.count_labels(labels)
        idx = 0
        print("Before OverSample")  #
        print(counts)  #

        overSampleAmount = self.div0(1 * max(counts), counts)  # ignore zeros in counts
        overSampleArraySize = int(sum(np.array(overSampleAmount, int) * counts))
        print(np.array(overSampleAmount, int) * counts)
        print(np.array(overSampleAmount, int))
        print(overSampleArraySize, len(labels))
        imagesOverSampled = np.zeros((overSampleArraySize, int(self.nw)), np.float16)
        labelsOverSampled = np.zeros(overSampleArraySize, np.uint16)
        filenamesOverSampled = np.empty(overSampleArraySize, dtype=object)
        typeNamesOverSampled = np.empty(overSampleArraySize, dtype=object)

        counts1 = np.zeros(self.nLabels)

        imagesShuf, labelsShuf, filenamesShuf, typeNamesShuf = self.shuffle_arrays(images, labels, filenames, typeNames)
        print(len(labelsShuf))
        for i in range(len(labelsShuf)):
            label = labelsShuf[i]
            image = imagesShuf[i]
            filename = filenamesShuf[i]
            typeName = typeNamesShuf[i]

            labelIndex = label # np.argmax(label)
            
            print(idx, i, int(overSampleAmount[labelIndex]))
            if overSampleAmount[labelIndex] < 10:
                std = 0.03
            else:
                std = 0.05
            for r in range(int(overSampleAmount[labelIndex])):
                imagesOverSampled[idx] = self.augment_data(image, stdDevMean=0.05, stdDevStdDev=std)  # image
                labelsOverSampled[idx] = label
                filenamesOverSampled[idx] = filename
                typeNamesOverSampled[idx] = typeName
                counts1[labelIndex] += 1
                idx += 1

        print("After OverSample")  #
        print(counts1)  #

        print("Before Shuffling")
        imagesOverSampledShuf, labelsOverSampledShuf, filenamesOverSampledShuf, typeNamesOverSampledShuf = self.shuffle_arrays(imagesOverSampled, labelsOverSampled, filenamesOverSampled, typeNamesOverSampled)
        print("After Shuffling")
        return imagesOverSampledShuf, labelsOverSampledShuf, filenamesOverSampledShuf, typeNamesOverSampledShuf


class CreateArrays(object):
    def __init__(self, w0, w1, nw, nTypes, minAge, maxAge, ageBinSize, typeList, minZ, maxZ, redshiftPrecision, hostTypes=None, nHostTypes=None):
        self.w0 = w0
        self.w1 = w1
        self.nw = nw
        self.nTypes = nTypes
        self.minAge = minAge
        self.maxAge = maxAge
        self.ageBinSize = ageBinSize
        self.typeList = typeList
        self.minZ = minZ
        self.maxZ = maxZ
        self.numOfRedshifts = (maxZ - minZ) * 1./redshiftPrecision
        self.ageBinning = AgeBinning(minAge, maxAge, ageBinSize)
        self.numOfAgeBins = self.ageBinning.age_bin(maxAge-0.1) + 1
        self.nLabels = nTypes * self.numOfAgeBins * nHostTypes
        self.createLabels = CreateLabels(self.nTypes, self.minAge, self.maxAge, self.ageBinSize, self.typeList, hostTypes, nHostTypes)
        self.hostTypes = hostTypes

    def snid_templates_to_arrays(self, snidTemplateLocation, tempfilelist):
        """ This function is for the SNID processed files, which
            have been preprocessed to negatives, and so cannot be
            imaged yet """

        tempList = TempList().temp_list(tempfilelist) #Arbrirary redshift to read filelist
        typeList = []
        images = np.empty((0, int(self.nw)), np.float16)  # Number of pixels
        labelsIndexes = [] # labels = np.empty((0, self.nLabels), np.uint8)  # Number of labels (SN types)
        filenames = []#np.empty(0)
        typeNames = []#np.empty(0)
        agesList = []

        for i in range(0, len(tempList)):
            ncols = 15
            readSpectra = ReadSpectra(self.w0, self.w1, self.nw, snidTemplateLocation + tempList[i])
            
            for ageidx in range(0, 1000):
                if ageidx < ncols:
                    for z in np.linspace(self.minZ, self.maxZ, self.numOfRedshifts + 1):
                        tempwave, tempflux, ncols, ages, ttype, tminindex, tmaxindex = readSpectra.snid_template_data(ageidx, z)
                        agesList.append(ages[ageidx])
                        if not tempflux.any():
                            print("NO DATA for {} ageIdx:{} z>={}".format(tempList[i], ageidx, z))
                            break

                        if self.minAge < float(ages[ageidx]) < self.maxAge:
                            labelIndex, typeName = self.createLabels.label_array(ttype, ages[ageidx])
                            nonzeroflux = tempflux[tminindex:tmaxindex + 1]
                            newflux = (nonzeroflux - min(nonzeroflux)) / (max(nonzeroflux) - min(nonzeroflux))
                            newflux2 = np.concatenate((tempflux[0:tminindex], newflux, tempflux[tmaxindex + 1:]))
                            images = np.append(images, np.array([newflux2]), axis=0)  # images.append(newflux2)
                            labelsIndexes.append(labelIndex) # labels = np.append(labels, np.array([label]), axis=0)  # labels.append(ttype)
                            filenames.append(tempList[i] + '_' + ttype + '_' + str(ages[ageidx]) + '_z' + str(z))
                            typeNames.append(typeName)

                    print(tempList[i], ageidx, ncols)
                else:
                    break

            # Create List of all SN types
            if ttype not in typeList:
                typeList.append(ttype)
        print(len(images))

        try:
            print("SIZE OF ARRAYS:")
            print(images.nbytes)
            print(np.array(labelsIndexes).nbytes)
            print(np.array(filenames).nbytes)
            print(np.array(typeNames).nbytes)
        except:
            print("Exception Raised")

        return typeList, images, np.array(labelsIndexes), np.array(filenames), np.array(typeNames)

    def combined_sn_gal_templates_to_arrays(self, snTemplateLocation, snTempFileList, galTemplateLocation, galTempList):
        snTempList = TempList().temp_list(snTempFileList)
        typeList = []
        images = np.empty((0, int(self.nw)), np.float16)  # Number of pixels
        labelsIndexes = [] # labels = np.empty((0, self.nLabels), np.uint8)  # Number of labels (SN types)
        filenames = []  # np.empty(0)
        typeNames = []  # np.empty(0)
        agesList = []
        for j in range(len(galTempList)):
            for i in range(0, len(snTempList)):
                ncols = 15
                readSpectra = ReadSpectra(self.w0, self.w1, self.nw, snTemplateLocation + snTempList[i], galTemplateLocation + galTempList[j])
                for ageidx in range(0, 1000):
                    if ageidx < ncols:
                        for snCoeff in [0.01, 0.02, 0.05, 0.07, 0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9]:
                            galCoeff = 1 - snCoeff
                            for z in np.linspace(self.minZ, self.maxZ, self.numOfRedshifts + 1):
                                tempwave, tempflux, ncols, ages, ttype, tminindex, tmaxindex = readSpectra.sn_plus_gal_template(ageidx, snCoeff, galCoeff, z)
                                agesList.append(ages[ageidx])
                                if not tempflux.any():
                                    print("NO DATA for {} {} ageIdx:{} z>={}".format(galTempList[j], snTempList[i], ageidx, z))
                                    break

                                if self.minAge < float(ages[ageidx]) < self.maxAge:
                                    if self.hostTypes is None: # Checks if we are classifying by host as well
                                        labelIndex, typeName = self.createLabels.label_array(ttype, ages[ageidx], host=None)
                                    else:
                                        labelIndex, typeName = self.createLabels.label_array(ttype, ages[ageidx], host=galTempList[j])
                                    nonzeroflux = tempflux[tminindex:tmaxindex + 1]
                                    newflux = (nonzeroflux - min(nonzeroflux)) / (max(nonzeroflux) - min(nonzeroflux))
                                    newflux2 = np.concatenate((tempflux[0:tminindex], newflux, tempflux[tmaxindex + 1:]))
                                    images = np.append(images, np.array([newflux2]), axis=0)
                                    labelsIndexes.append(labelIndex) # labels = np.append(labels, np.array([label]), axis=0)
                                    filenames.append("{0}_{1}_{2}_{3}_snCoeff{4}_z{5}".format(snTempList[i], ttype, str(ages[ageidx]), galTempList[j], snCoeff, (z)))
                                    typeNames.append(typeName)

                        print(snTempList[i], ageidx, ncols, galTempList[j], snCoeff)
                    else:
                        break

                # Create List of all SN types
                if ttype not in typeList:
                    typeList.append(ttype)
        print(len(images))

        try:
            print("SIZE OF ARRAYS:")
            print(images.nbytes)
            print(np.array(labelsIndexes).nbytes)
            print(np.array(filenames).nbytes)
            print(np.array(typeNames).nbytes)
        except:
            print("Exception Raised")

        return typeList, images, np.array(labelsIndexes), np.array(filenames), np.array(typeNames)

    def combined_sn_gal_arrays_multiprocessing(self, snTemplateLocation, snTempFileList, galTemplateLocation, galTempFileList):
        galTempList = TempList().temp_list(galTempFileList)

        images = np.empty((0, int(self.nw)), np.float16)
        labelsIndexes = np.empty(0, np.uint16)
        filenames = np.empty(0)
        typeNames = np.empty(0)

        t1 = time.time()
        pool = mp.Pool(processes=11)
        results = [pool.apply_async(self.combined_sn_gal_templates_to_arrays, args=(snTemplateLocation, snTempFileList, galTemplateLocation, [gList],)) for gList in galTempList]
        pool.close()
        pool.join()

        outputs = [p.get() for p in results]

        for out in outputs:
            typeList, imagesPart, labelsPart, filenamesPart, typeNamesPart = out
            images = np.append(images, imagesPart, axis=0)
            labelsIndexes = np.append(labelsIndexes, labelsPart, axis=0)
            filenames = np.append(filenames, filenamesPart)
            typeNames = np.append(typeNames, typeNamesPart)

        t2 = time.time()
        print("time spent: {0:.2f}".format(t2 - t1))

        try:
            print("SIZE OF ARRAYS:")
            print(images.nbytes)
            print(labelsIndexes.nbytes)
            print(filenames.nbytes)
            print(typeNames.nbytes)
        except:
            print("Exception Raised")
        print("Completed Creating Arrays!")

        return typeList, images, labelsIndexes, filenames, typeNames



import os
import dash

directory = r'/home/daniel/Documents/DES_Y4_Spectra/'

# # LIST ALL SPECTRAL FILENAMES FROM DROPBOX
# filenames = []
# extensions = ('.flm', '.dat', '.txt', '.fits')
# for path, subdirs, files in os.walk(directory):
#     for name in files:
#         if name.lower().endswith(extensions):
#             filenames.append(os.path.join(path, name).replace(directory, ''))
# filenames.sort()
# for f in filenames:
#     print("('%s', 0)," % f)

#Convert files to fits

#Get redshifts from MARZ


filenames = [os.path.join(directory, i[0]) for i in atels]
knownRedshifts = [i[1] for i in atels]

classification = dash.Classify(filenames, knownRedshifts)
bestFits, bestTypes, rejectionLabels = classification.list_best_matches(n=5)

# SAVE BEST MATCHES
print bestFits
f = open('classification_results.txt', 'w')
for i in range(len(atels)):
    f.write("%s   %s     %s      %s\n %s\n\n" % (atels[i][0], atels[i][1], bestTypes[i], rejectionLabels[i], bestFits[i]))
f.close()
print("Finished classifying %d spectra!" % len(atels))

# PLOT SPECTRUM ON GUI
# classification.plot_with_gui(indexToPlot=0)




import numpy as np
from scipy.fftpack import fft
import matplotlib.pyplot as plt


def cross_correlation(inputFlux, tempFlux, nw):
    inputFourier = fft(inputFlux)
    tempFourier = fft(tempFlux)
    # kinput = 2 * np.pi / inputwave
    # ktemp = 2 * np.pi / tempwave

    product = inputFourier * np.conj(tempFourier)
    xCorr = fft(product)

    rmsInput = np.std(inputFourier)
    rmsTemp = np.std(tempFourier)

    xCorrNorm = (1. / (nw * rmsInput * rmsTemp)) * xCorr

    rmsXCorr = np.std(product)

    xCorrNormRearranged = np.concatenate((xCorrNorm[int(len(xCorrNorm) / 2):], xCorrNorm[0:int(len(xCorrNorm) / 2)]))

    # return xCorr, rmsInput, rmsTemp, xCorrNorm, rmsXCorr, xCorrNormRearranged
    return xCorrNormRearranged


def calc_redshift_from_crosscorr(crossCorr, nw, dwlog):
    deltaPeak = np.argmax(abs(crossCorr))

    # z = np.exp(deltaPeak * dwlog) - 1 #equation 13 of Blondin)
    zAxisIndex = np.concatenate((np.arange(-nw / 2, 0), np.arange(0, nw / 2)))
    if deltaPeak < nw / 2:
        z = (np.exp(abs(zAxisIndex) * dwlog) - 1)[deltaPeak]
    else:
        z = -(np.exp(abs(zAxisIndex) * dwlog) - 1)[deltaPeak]

    return z, crossCorr


def get_redshift_axis(nw, dwlog):
    zAxisIndex = np.concatenate((np.arange(-nw / 2, 0), np.arange(0, nw / 2)))
    zAxis = np.zeros(nw)
    zAxis[0:int(nw / 2 - 1)] = -(np.exp(abs(zAxisIndex[0:int(nw / 2 - 1)]) * dwlog) - 1)
    zAxis[int(nw / 2):] = (np.exp(abs(zAxisIndex[int(nw / 2):]) * dwlog) - 1)
    zAxis = zAxis[::-1]

    return zAxis


def get_redshift(inputFlux, tempFlux, nw, dwlog):
    crossCorr = cross_correlation(inputFlux, tempFlux, nw)
    redshift, crossCorr = calc_redshift_from_crosscorr(crossCorr, nw, dwlog)

    return redshift, crossCorr


def get_median_redshift(inputFlux, tempFluxes, nw, dwlog):
    redshifts = []
    crossCorrs = []
    for tempFlux in tempFluxes:
        redshift, crossCorr = get_redshift(inputFlux, tempFlux, nw, dwlog)
        redshifts.append(redshift)
        crossCorrs.append(crossCorr)

    if redshifts != []:
        medianIndex = np.argsort(redshifts)[len(redshifts)//2]
        medianRedshift = redshifts[medianIndex]
    else:
        return None, None

    return medianRedshift, np.abs(crossCorrs[medianIndex])



#Pre-processing class

from scipy.signal import medfilt
import numpy as np
from dash.preprocessing import ReadSpectrumFile, PreProcessSpectrum, ProcessingTools


def limit_wavelength_range(wave, flux, minWave, maxWave):
    minIdx = (np.abs(wave-minWave)).argmin()
    maxIdx = (np.abs(wave-maxWave)).argmin()

    flux[:minIdx] = np.zeros(minIdx)
    flux[maxIdx:] = np.zeros(len(flux)-maxIdx)

    return flux


class PreProcessing(object):
    """ Pre-processes spectra for cross correlation """
    
    def __init__(self, filename, w0, w1, nw):
        self.filename = filename
        self.w0 = w0
        self.w1 = w1
        self.nw = nw
        self.numSplinePoints = 13
        self.processingTools = ProcessingTools()
        self.readSpectrumFile = ReadSpectrumFile(filename, w0, w1, nw)
        self.preProcess = PreProcessSpectrum(w0, w1, nw)

        self.spectrum = self.readSpectrumFile.file_extension()

    def galaxy_template(self, z):
        self.wave, self.flux = self.spectrum
        wave, flux = self.readSpectrumFile.two_col_input_spectrum(self.wave, self.flux, z)
        binnedwave, binnedflux, minindex, maxindex = self.preProcess.log_wavelength(wave, flux)
        newflux, continuum = self.preProcess.continuum_removal(binnedwave, binnedflux, self.numSplinePoints, minindex, maxindex)
        meanzero = self.preProcess.mean_zero(binnedwave, newflux, minindex, maxindex)
        apodized = self.preProcess.apodize(binnedwave, meanzero, minindex, maxindex)

        plt.figure('Galaxy_SB1')
        plt.plot(wave,flux, label='original')
        plt.plot(binnedwave,binnedflux, label='binned')
        plt.plot(binnedwave, continuum, label='continuum')
        plt.plot(binnedwave,newflux, label='continuumSubtracted')
        plt.plot(binnedwave, meanzero, label='meanzero')
        plt.plot(binnedwave,apodized, label='apodized')
        plt.legend()
        #plt.show()

        return binnedwave, apodized, minindex, maxindex


    def two_column_data(self, z, smooth, minWave, maxWave):
        self.wave, self.flux = self.spectrum
        self.flux = limit_wavelength_range(self.wave, self.flux, minWave, maxWave)
        self.wDensity = (self.w1 - self.w0)/self.nw  # Average wavelength spacing
        wavelengthDensity = (max(self.wave) - min(self.wave)) / len(self.wave)

        filterSize = int(self.wDensity / wavelengthDensity * smooth / 2) * 2 + 1
        preFiltered = medfilt(self.flux, kernel_size=filterSize)
        wave, flux = self.readSpectrumFile.two_col_input_spectrum(self.wave, preFiltered, z)
        binnedwave, binnedflux, minindex, maxindex = self.preProcess.log_wavelength(wave, flux)
        newflux, continuum = self.preProcess.continuum_removal(binnedwave, binnedflux, self.numSplinePoints, minindex, maxindex)
        meanzero = self.preProcess.mean_zero(binnedwave, newflux, minindex, maxindex)
        apodized = self.preProcess.apodize(binnedwave, meanzero, minindex, maxindex)


        #filterSize = smooth * 2 + 1
        medianFiltered = medfilt(apodized, kernel_size=1)#filterSize)


        # from scipy.interpolate import interp1d
        #
        # plt.plot(self.flux)
        #
        # spline = interp1d(binnedwave[minindex:maxindex], binnedflux[minindex:maxindex], kind='cubic')
        # waveSpline = np.linspace(binnedwave[minindex],binnedwave[maxindex-1],num=self.numSplinePoints)
        # print(spline)
        # print('###')
        # print(spline(binnedwave[minindex:maxindex]))
        # plt.figure('1')
        # plt.plot(waveSpline, spline(waveSpline), '--', label='spline')
        #
        # print(wave)
        # print(binnedwave)
        # print(binnedflux)
        # print(len(binnedwave))
        # plt.plot(wave,flux)
        # plt.figure('2')
        # plt.plot(binnedwave, binnedflux, label='binned')
        # plt.plot(binnedwave, newflux, label='continuumSubtract1')
        # plt.plot(binnedwave, continuum, label='polyfit1')
        # print(len(binnedwave))
        # print((min(binnedwave), max(binnedwave)))
        # print(len(newflux))
        #
        # #newflux2, poly2 = self.preProcess.continuum_removal(binnedwave, binnedflux, 6, minindex, maxindex)
        # #plt.plot(binnedwave, newflux2, label='continuumSubtract2')
        # #plt.plot(binnedwave, poly2, label='polyfit2')
        # plt.plot(binnedwave, apodized, label='taper')
        # plt.legend()
        # plt.figure('filtered')
        # plt.plot(medianFiltered)
        # plt.figure('3')
        # plt.plot(medfilt(apodized,kernel_size=3))
        # plt.show()

        return binnedwave, medianFiltered, minindex, maxindex

    def snid_template_data(self, ageIdx, z):
        """lnw templates """
        wave, fluxes, ncols, ages, ttype, splineInfo = self.spectrum
        wave, flux = self.processingTools.redshift_spectrum(wave, fluxes[ageIdx], z)
        binnedwave, binnedflux, minindex, maxindex = self.preProcess.log_wavelength(wave, flux)
        medianFiltered = medfilt(binnedflux, kernel_size=1)

        return binnedwave, medianFiltered, ncols, ages, ttype, minindex, maxindex

# if __name__ == '__main__':
#     # fData = '/home/dan/Desktop/SNClassifying_Pre-alpha/templates/superfit_templates/sne/Ia/sn2002bo.m01.dat'
#     # preData = PreProcessing(fGal, 2500, 10000, 1024)
#     # waveData,fluxData,minIData,maxIData = preData.two_column_data(0, 0)
#
#     fGal = '/home/dan/Desktop/SNClassifying_Pre-alpha/templates/galaxy_templates/GalaxyTemplates/Sa'
#     preGal = PreProcessing(fGal, 2500, 10000, 1024)
#     waveGal,fluxGal,minIGal,maxIGal = preGal.galaxy_template(0)
#
#     fSN = '/home/dan/Desktop/SNClassifying_Pre-alpha/templates/snid_templates_Modjaz_BSNIP/sn2001br.lnw'
#     preSN = PreProcessing(fSN, 2500, 10000, 1024)
#     waveSN, fluxSN, ncols, ages, ttype, minISN, maxISN = preSN.snid_template_data(0, 0)
#
#     plt.figure('SN and Galaxy Template')
#     minI = max(minIGal, minISN)
#     maxI = min(maxIGal, maxISN)
#     plt.axvline(x=waveSN[minI], color='k', linestyle='--')
#     plt.axvline(x=waveSN[maxI], color='k', linestyle='--')
#
#
#
#     plt.plot(waveGal,fluxGal, label='galaxy')
#     plt.plot(waveSN,fluxSN, label='SN')
#
#     waveSN = waveSN[minI:maxI]
#     fluxSN = fluxSN[minI:maxI]
#     fluxGal = fluxGal[minI:maxI]
#
#     plt.plot(waveSN,fluxGal+fluxSN, label='Added-50%')
#     plt.legend()
#     plt.show()



from dash.preprocessing import ReadSpectrumFile, ProcessingTools, PreProcessSpectrum
from dash.array_tools import zero_non_overlap_part, normalise_spectrum
import matplotlib.pyplot as plt
import numpy as np


class CombineSnAndHost(object):
    def __init__(self, snFile, galFile, w0, w1, nw):
        self.w0 = w0
        self.w1 = w1
        self.nw = nw
        self.numSplinePoints = 13
        self.processingTools = ProcessingTools()
        self.snReadSpectrumFile = ReadSpectrumFile(snFile, w0, w1, nw)
        self.galReadSpectrumFile = ReadSpectrumFile(galFile, w0, w1, nw)
        self.snSpectrum = self.snReadSpectrumFile.file_extension()
        self.snWave, self.snFluxes, self.ncols, self.ages, self.ttype, self.splineInfo = self.snSpectrum
        self.galSpectrum = self.galReadSpectrumFile.file_extension()
        self.preProcess = PreProcessSpectrum(w0, w1, nw)

    def snid_sn_template_data(self, ageIdx):
        # Undo continuum in the following step in preprocessing.py
        wave, flux = self.snReadSpectrumFile.snid_template_undo_processing(self.snWave, self.snFluxes[ageIdx], self.splineInfo, ageIdx)

        binnedWave, binnedFlux, minIndex, maxIndex = self.preProcess.log_wavelength(wave, flux)
        binnedFluxNorm = normalise_spectrum(binnedFlux)

        return binnedWave, binnedFluxNorm, minIndex, maxIndex

    def gal_template_data(self):
        wave, flux = self.galSpectrum

        # Limit bounds from w0 to w1 and normalise flux
        wave, flux = self.galReadSpectrumFile.two_col_input_spectrum(wave, flux, z=0)
        binnedWave, binnedFlux, minIndex, maxIndex = self.preProcess.log_wavelength(wave, flux)
        binnedFluxNorm = normalise_spectrum(binnedFlux)

        return binnedWave, binnedFluxNorm, minIndex, maxIndex

    def overlapped_spectra(self, snAgeIdx):
        snWave, snFlux, snMinIndex, snMaxIndex = self.snid_sn_template_data(snAgeIdx)
        galWave, galFlux, galMinIndex, galMaxIndex = self.gal_template_data()

        minIndex = max(snMinIndex, galMinIndex)
        maxIndex = min(snMaxIndex, galMaxIndex)

        snFlux = zero_non_overlap_part(snFlux, minIndex, maxIndex)
        galFlux = zero_non_overlap_part(galFlux, minIndex, maxIndex)

        return snWave, snFlux, galWave, galFlux, minIndex, maxIndex

    def sn_plus_gal(self, snCoeff, galCoeff, snAgeIdx):
        snWave, snFlux, galWave, galFlux, minIndex, maxIndex = self.overlapped_spectra(snAgeIdx)

        combinedFlux = (snCoeff * snFlux) + (galCoeff * galFlux)

        return snWave, combinedFlux, minIndex, maxIndex

    def training_template_data(self, snAgeIdx, snCoeff, galCoeff, z):
        wave, flux, minIndex, maxIndex = self.sn_plus_gal(snCoeff, galCoeff, snAgeIdx)
        wave, flux = self.processingTools.redshift_spectrum(wave, flux, z)
        binnedWave, binnedFlux, minIndex, maxIndex = self.preProcess.log_wavelength(wave, flux)
        newFlux, continuum = self.preProcess.continuum_removal(binnedWave, binnedFlux, self.numSplinePoints, minIndex, maxIndex)
        meanZero = self.preProcess.mean_zero(binnedWave, newFlux, minIndex, maxIndex)
        apodized = self.preProcess.apodize(binnedWave, meanZero, minIndex, maxIndex)
        # fluxNorm = self._normalise_spectrum(apodized) # This happens in create_arrays anyway
        # Could  median filter here, but trying without it now

        return binnedWave, apodized, minIndex, maxIndex, self.ncols, self.ages, self.ttype



if __name__ == '__main__':
    fSN = '/Users/danmuth/PycharmProjects/DASH/templates/snid_templates_Modjaz_BSNIP/sn2001br.lnw'
    fGal = '/Users/danmuth/PycharmProjects/DASH/templates/superfit_templates/gal/Sa'
    combine = CombineSnAndHost(fSN, fGal, 2500, 10000, 1024)

    f = plt.figure()
    xSN, ySN, minSN, maxSN = combine.snid_sn_template_data(ageIdx=0)
    xGal, yGal, minGal, maxGal = combine.gal_template_data()
    plt.plot(xSN, ySN, 'b')
    plt.plot(xGal, yGal, 'r')

    f2 = plt.figure()
    xSN, ySN, xGal, yGal, minI, maxI = combine.overlapped_spectra(0)
    xCombined, yCombined, minI, maxI = combine.sn_plus_gal(0.5, 0.5, 0)
    xTemp, ytemp, minI, maxI, NCOL, AGE, TTYPE = combine.training_template_data(0, 0.5, 0.5, 0)
    plt.plot(xSN, ySN, 'b')
    plt.plot(xGal, yGal, 'r')
    plt.plot(xCombined, yCombined, 'g')
    plt.plot(xTemp, ytemp, 'c')


    # galNames = ['E', 'S0', 'Sa', 'Sb', 'Sc', 'SB1', 'SB2', 'SB3', 'SB4', 'SB5', 'SB6',]
    # fGals = ['/Users/danmuth/PycharmProjects/DASH/templates/superfit_templates/gal/E',
    #         '/Users/danmuth/PycharmProjects/DASH/templates/superfit_templates/gal/S0',
    #         '/Users/danmuth/PycharmProjects/DASH/templates/superfit_templates/gal/Sa',
    #         '/Users/danmuth/PycharmProjects/DASH/templates/superfit_templates/gal/Sb',
    #         '/Users/danmuth/PycharmProjects/DASH/templates/superfit_templates/gal/Sc',
    #         '/Users/danmuth/PycharmProjects/DASH/templates/superfit_templates/gal/SB1',
    #         '/Users/danmuth/PycharmProjects/DASH/templates/superfit_templates/gal/SB2',
    #         '/Users/danmuth/PycharmProjects/DASH/templates/superfit_templates/gal/SB3',
    #         '/Users/danmuth/PycharmProjects/DASH/templates/superfit_templates/gal/SB4',
    #         '/Users/danmuth/PycharmProjects/DASH/templates/superfit_templates/gal/SB5',
    #         '/Users/danmuth/PycharmProjects/DASH/templates/superfit_templates/gal/SB6',]
    #
    # for i in range(len(fGals)):
    #     f3 = plt.figure()
    #     combine = CombineSnAndHost(fSN, fGals[i], 2500, 10000, 1024)
    #     xGal, yGal, minGal, maxGal = combine.gal_template_data()
    #     plt.plot(xGal, yGal)
    #     plt.savefig("%s.png" % galNames[i])

    plt.show()







import pickle
import numpy as np
from dash.create_arrays import AgeBinning, CreateLabels, ArrayTools, CreateArrays
import zipfile
import gzip
import os


class CreateTrainingSet(object):

    def __init__(self, snidTemplateLocation, snidTempFileList, w0, w1, nw, nTypes, minAge, maxAge, ageBinSize, typeList, minZ, maxZ, redshiftPrecision, galTemplateLocation, galTempFileList, hostTypes, nHostTypes):
        self.snidTemplateLocation = snidTemplateLocation
        self.snidTempFileList = snidTempFileList
        self.galTemplateLocation = galTemplateLocation
        self.galTempFileList = galTempFileList
        self.w0 = w0
        self.w1 = w1
        self.nw = nw
        self.nTypes = nTypes
        self.minAge = minAge
        self.maxAge = maxAge
        self.ageBinSize = ageBinSize
        self.typeList = typeList
        self.ageBinning = AgeBinning(self.minAge, self.maxAge, self.ageBinSize)
        self.numOfAgeBins = self.ageBinning.age_bin(self.maxAge-0.1) + 1
        self.nLabels = self.nTypes * self.numOfAgeBins * nHostTypes
        self.createArrays = CreateArrays(w0, w1, nw, nTypes, minAge, maxAge, ageBinSize, typeList, minZ, maxZ, redshiftPrecision, hostTypes, nHostTypes)
        self.arrayTools = ArrayTools(self.nLabels, self.nw)
        
    def type_amounts(self, labels):
        counts = self.arrayTools.count_labels(labels)

        return counts

    def all_templates_to_arrays(self):
        if self.galTemplateLocation is None or self.galTempFileList is None:
            snTypeList, images, labels, filenames, typeNames = self.createArrays.snid_templates_to_arrays(self.snidTemplateLocation, self.snidTempFileList)
        else:
            snTypeList, images, labels, filenames, typeNames = self.createArrays.combined_sn_gal_arrays_multiprocessing(self.snidTemplateLocation, self.snidTempFileList, self.galTemplateLocation, self.galTempFileList)

        imagesShuf, labelsShuf, filenamesShuf, typeNamesShuf = self.arrayTools.shuffle_arrays(images, labels, filenames, typeNames)

        typeAmounts = self.type_amounts(labels)
        
        return snTypeList, imagesShuf, labelsShuf, filenamesShuf, typeNamesShuf, typeAmounts

    def sort_data(self):
        trainPercentage = 0.9
        testPercentage = 0.1
        validatePercentage = 0.

        typeList, images, labels, filenames, typeNames, typeAmounts = self.all_templates_to_arrays()

        trainSize = int(trainPercentage * len(images))
        testSize = int(testPercentage * len(images))

        trainImages = images[:trainSize]
        testImages = images[trainSize: trainSize + testSize]
        validateImages = images[trainSize + testSize:]
        trainLabels = labels[:trainSize]
        testLabels = labels[trainSize: trainSize + testSize]
        validateLabels = labels[trainSize + testSize:]
        trainFilenames = filenames[:trainSize]
        testFilenames = filenames[trainSize: trainSize + testSize]
        validateFilenames = filenames[trainSize + testSize:]
        trainTypeNames = typeNames[:trainSize]
        testTypeNames = typeNames[trainSize: trainSize + testSize]
        validateTypeNames = typeNames[trainSize + testSize:]

        trainImagesOverSample, trainLabelsOverSample, trainFilenamesOverSample, trainTypeNamesOverSample = self.arrayTools.over_sample_arrays(trainImages, trainLabels, trainFilenames, trainTypeNames)
        testImagesShortlist, testLabelsShortlist, testFilenamesShortlist, testTypeNamesShortlist = testImages, testLabels, testFilenames, testTypeNames  # (testImages, testLabels, testFilenames)

        typeAmountsOverSampled = self.type_amounts(trainLabelsOverSample)

        return ((trainImagesOverSample, trainLabelsOverSample, trainFilenamesOverSample, trainTypeNamesOverSample),
                (testImagesShortlist, testLabelsShortlist, testFilenamesShortlist, testTypeNamesShortlist),
                (validateImages, validateLabels, validateFilenames, validateTypeNames),
                (typeAmounts, typeAmountsOverSampled))


class SaveTrainingSet(object):
    def __init__(self, snidTemplateLocation, snidTempFileList, w0, w1, nw, nTypes, minAge, maxAge, ageBinSize, typeList, minZ, maxZ, redshiftPrecision, galTemplateLocation=None, galTempFileList=None, hostTypes=None, nHostTypes=1):
        self.snidTemplateLocation = snidTemplateLocation
        self.snidTempFileList = snidTempFileList
        self.w0 = w0
        self.w1 = w1
        self.nw = nw
        self.nTypes = nTypes
        self.minAge = minAge
        self.maxAge = maxAge
        self.ageBinSize = ageBinSize
        self.typeList = typeList
        self.createLabels = CreateLabels(nTypes, minAge, maxAge, ageBinSize, typeList, hostTypes, nHostTypes)
        
        self.createTrainingSet = CreateTrainingSet(snidTemplateLocation, snidTempFileList, w0, w1, nw, nTypes, minAge, maxAge, ageBinSize, typeList, minZ, maxZ, redshiftPrecision, galTemplateLocation, galTempFileList, hostTypes, nHostTypes)
        self.sortData = self.createTrainingSet.sort_data()
        self.trainImages = self.sortData[0][0]
        self.trainLabels = self.sortData[0][1]
        self.trainFilenames = self.sortData[0][2]
        self.trainTypeNames = self.sortData[0][3]
        self.testImages = self.sortData[1][0]
        self.testLabels = self.sortData[1][1]
        self.testFilenames = self.sortData[1][2]
        self.testTypeNames = self.sortData[1][3]
        self.validateImages = self.sortData[2][0]
        self.validateLabels = self.sortData[2][1]
        self.validateFilenames = self.sortData[2][2]
        self.validateTypeNames = self.sortData[2][3]
        self.typeAmounts = self.sortData[3][0]
        self.typeAmountsOverSampled = self.sortData[3][1]

        self.typeNamesList = self.createLabels.type_names_list()

    def type_amounts(self):
        for i in range(len(self.typeNamesList)):
            print(str(self.typeAmounts[i]) + ": " + str(self.typeNamesList[i]))
        return self.typeNamesList, self.typeAmounts

    def save_arrays(self, saveFilename):
        saveFilename = saveFilename
        arraysToSave = {'trainImages.npy.gz': self.trainImages, 'trainLabels.npy.gz': self.trainLabels,
                        'testImages.npy.gz': self.testImages, 'testLabels.npy.gz': self.testLabels,
                        'testTypeNames.npy.gz': self.testTypeNames, 'typeNamesList.npy.gz': self.typeNamesList,
                        'trainFilenames.npy.gz': self.trainFilenames, 'trainTypeNames.npy.gz': self.trainTypeNames}

        try:
            print("SIZE OF ARRAYS TRAINING:")
            print(self.trainImages.nbytes, self.testImages.nbytes)
            print(self.trainLabels.nbytes, self.testLabels.nbytes)
            print(self.trainFilenames.nbytes, self.testFilenames.nbytes)
            print(self.trainTypeNames.nbytes, self.testTypeNames.nbytes)
        except:
            print("Exception Raised")

        for filename, array in arraysToSave.items():
            f = gzip.GzipFile(filename, "w")
            np.save(file=f, arr=array)
            f.close()

        with zipfile.ZipFile(saveFilename, 'w') as myzip:
            for f in arraysToSave.keys():
                myzip.write(f)

        print("Saved Training Set to: " + saveFilename)

        # Delete npy.gz files
        for filename in arraysToSave.keys():
            os.remove(filename)


def create_training_set_files(dataDirName, minZ=0, maxZ=0, redshiftPrecision=0.01, trainWithHost=True, classifyHost=False):
    with open(dataDirName + 'training_params.pickle', 'rb') as f1:
        pars = pickle.load(f1)
    nTypes, w0, w1, nw, minAge, maxAge, ageBinSize, typeList = pars['nTypes'], pars['w0'], pars['w1'], \
                                                                         pars['nw'], pars['minAge'], pars['maxAge'], \
                                                                         pars['ageBinSize'], pars['typeList']
    hostList, nHostTypes = None, 1

    scriptDirectory = os.path.dirname(os.path.abspath(__file__))

    snidTemplateLocation = os.path.join(scriptDirectory, "../templates/snid_templates_Modjaz_BSNIP/")
    snidTempFileList = snidTemplateLocation + 'templist.txt'
    if trainWithHost:
        galTemplateLocation = os.path.join(scriptDirectory, "../templates/superfit_templates/gal/")
        galTempFileList = galTemplateLocation + 'gal.list'
        if classifyHost:
            hostList = pars['galTypeList']
            nHostTypes = len(hostList)
    else:
        galTemplateLocation, galTempFileList = None, None

    saveTrainingSet = SaveTrainingSet(snidTemplateLocation, snidTempFileList, w0, w1, nw, nTypes, minAge, maxAge, ageBinSize, typeList, minZ, maxZ, redshiftPrecision, galTemplateLocation, galTempFileList, hostList, nHostTypes)
    typeNamesList, typeAmounts = saveTrainingSet.type_amounts()

    saveFilename = dataDirName + 'training_set.zip'
    saveTrainingSet.save_arrays(saveFilename)

    return saveFilename


if __name__ == '__main__':
    trainingSetFilename = create_training_set_files('data_files/', minZ=0, maxZ=0, redshiftPrecision=0.01, trainWithHost=False, classifyHost=False)



import numpy as np
import os
from dash.preprocessing import ProcessingTools, PreProcessSpectrum
from dash.array_tools import zero_non_overlap_part, normalise_spectrum
scriptDirectory = os.path.dirname(os.path.abspath(__file__))


def get_templates(snName, snAge, hostName, snTemplates, galTemplates, nw):
    snInfos = np.copy(snTemplates[snName][snAge]['snInfo'])
    snNames = np.copy(snTemplates[snName][snAge]['names'])
    if hostName != "No Host" and hostName != "":
        hostInfos = np.copy(galTemplates[hostName]['galInfo'])
        hostNames = np.copy(galTemplates[hostName]['names'])
    else:
        hostInfos = np.array([[np.zeros(nw), np.zeros(nw), 1, nw - 1]])
        hostNames = np.array(["No Host"])

    return snInfos, snNames, hostInfos, hostNames


def load_templates(templateFilename):
    loaded = np.load(os.path.join(scriptDirectory, templateFilename))
    snTemplates = loaded['snTemplates'][()]
    galTemplates = loaded['galTemplates'][()]

    return snTemplates, galTemplates


class ReadBinnedTemplates(object):
    def __init__(self, snInfo, galInfo, w0, w1, nw):
        self.snInfo = snInfo
        self.galInfo = galInfo
        self.processingTools = ProcessingTools()
        self.numSplinePoints = 13
        self.preProcess = PreProcessSpectrum(w0, w1, nw)

    def overlapped_spectra(self):
        snWave, snFlux, snMinIndex, snMaxIndex = self.snInfo
        galWave, galFlux, galMinIndex, galMaxIndex = self.galInfo

        minIndex = max(snMinIndex, galMinIndex)
        maxIndex = min(snMaxIndex, galMaxIndex)

        snFlux = zero_non_overlap_part(snFlux, minIndex, maxIndex)
        galFlux = zero_non_overlap_part(galFlux, minIndex, maxIndex)

        return snWave, snFlux, galWave, galFlux, minIndex, maxIndex

    def sn_plus_gal(self, snCoeff, galCoeff):
        snWave, snFlux, galWave, galFlux, minIndex, maxIndex = self.overlapped_spectra()

        combinedFlux = (snCoeff * snFlux) + (galCoeff * galFlux)

        return snWave, combinedFlux, minIndex, maxIndex

    def template_data(self, snCoeff, galCoeff, z):
        wave, flux, minIndex, maxIndex = self.sn_plus_gal(snCoeff, galCoeff)
        wave, flux = self.processingTools.redshift_spectrum(wave, flux, z)
        binnedWave, binnedFlux, minIndex, maxIndex = self.preProcess.log_wavelength(wave, flux)
        newFlux, continuum = self.preProcess.continuum_removal(binnedWave, binnedFlux, self.numSplinePoints, minIndex, maxIndex)
        meanZero = self.preProcess.mean_zero(binnedWave, newFlux, minIndex, maxIndex)
        apodized = self.preProcess.apodize(binnedWave, meanZero, minIndex, maxIndex)
        fluxNorm = normalise_spectrum(apodized)
        fluxNorm = zero_non_overlap_part(fluxNorm, minIndex, maxIndex)
        # Could  median filter here, but trying without it now

        return binnedWave, fluxNorm

if __name__ == "__main__":
    templateFilename1 = 'models/sn_and_host_templates.npz'
    snTemplates1, galTemplates1 = load_templates(templateFilename1)
    snInfoList = snTemplates1['Ia-norm']['-2 to 2']['snInfo']
    galInfoList = galTemplates1['S0']['galInfo']
    for i in range(len(snInfoList)):
        readBinnedTemplates = ReadBinnedTemplates(snInfoList[i], galInfoList[0], 2500, 10000, 1024)
        wave, flux = readBinnedTemplates.template_data(snCoeff=0.5, galCoeff=0.5, z=0)
        print(i)



import os
import numpy as np
import itertools
import tensorflow as tf
from dash.multilayer_convnet import convnet_variables
import zipfile
import gzip
import time
from dash.array_tools import labels_indexes_to_arrays
from dash.model_statistics import calc_model_statistics


def train_model(dataDirName):
    # Open training data files
    scriptDirectory = os.path.dirname(os.path.abspath(__file__))
    trainingSet = dataDirName + 'training_set.zip'
    extractedFolder = os.path.join(dataDirName, 'training_set')
    # zipRef = zipfile.ZipFile(trainingSet, 'r')
    # zipRef.extractall(extractedFolder)
    # zipRef.close()
    os.system("unzip %s -d %s" % (trainingSet, extractedFolder))

    npyFiles = {}
    fileList = os.listdir(extractedFolder)
    for filename in fileList:
        if filename.endswith('.gz'):
            f = os.path.join(scriptDirectory, extractedFolder, filename)
            # # npyFiles[filename.strip('.npy.gz')] = gzip.GzipFile(f, 'r')
            # gzFile = gzip.open(f, "rb")
            # unCompressedFile = open(f.strip('.gz'), "wb")
            # decoded = gzFile.read()
            # unCompressedFile.write(decoded)
            # gzFile.close()
            # unCompressedFile.close()
            npyFiles[filename.strip('.npy.gz')] = f.strip('.gz')
            os.system("gzip -dk %s" % f)

    trainImages = np.load(npyFiles['trainImages'], mmap_mode='r')
    trainLabels = np.load(npyFiles['trainLabels'], mmap_mode='r')
    testImages = np.load(npyFiles['testImages'], mmap_mode='r')
    testLabelsIndexes = np.load(npyFiles['testLabels'], mmap_mode='r')
    typeNamesList = np.load(npyFiles['typeNamesList'])
    testTypeNames = np.load(npyFiles['testTypeNames'])
    # testImages = np.load(npyFiles['testImagesNoGal'], mmap_mode='r')
    # testLabelsIndexes = np.load(npyFiles['testLabelsNoGal'], mmap_mode='r')

    print("Completed creatingArrays")
    print(len(trainImages))

    nLabels = len(typeNamesList)

    # Set up the convolutional network architecture
    N = 1024
    imWidth = 32  # Image size and width
    imWidthReduc = 8
    a = []
    x, y_, keep_prob, y_conv = convnet_variables(imWidth, imWidthReduc, N, nLabels)

    with tf.Session() as sess: # config=tf.ConfigProto(inter_op_parallelism_threads=1, intra_op_parallelism_threads=1)) as sess:
        # TRAIN AND EVALUATE MODEL
        cross_entropy = tf.reduce_mean(-tf.reduce_sum(y_ * tf.log(y_conv + 1e-8), reduction_indices=[1]))
        train_step = tf.train.AdamOptimizer(1e-4).minimize(cross_entropy)
        correct_prediction = tf.equal(tf.argmax(y_conv, 1), tf.argmax(y_, 1))
        accuracy = tf.reduce_mean(tf.cast(correct_prediction, tf.float32))

        sess.run(tf.global_variables_initializer())

        testLabels = labels_indexes_to_arrays(testLabelsIndexes[0:400], nLabels)
        testImages = testImages[0:400]
        # testLabelsWithGal = labels_indexes_to_arrays(testLabelsIndexesWithGal[0:200], nLabels)
        # testImagesWithGal = testImagesWithGal[0:200]

        trainImagesCycle = itertools.cycle(trainImages)
        trainLabelsCycle = itertools.cycle(trainLabels)
        for i in range(400000):
            batch_xs = np.array(list(itertools.islice(trainImagesCycle, 50 * i, 50 * i + 50)))
            batch_ys = labels_indexes_to_arrays(list(itertools.islice(trainLabelsCycle, 50 * i, 50 * i + 50)), nLabels)
            train_step.run(feed_dict={x: batch_xs, y_: batch_ys, keep_prob: 0.5})
            if i % 100 == 0:
                train_accuracy = accuracy.eval(feed_dict={x: batch_xs, y_: batch_ys, keep_prob: 1.0})
                print("step %d, training accuracy %g" % (i, train_accuracy))
                testAcc = accuracy.eval(feed_dict={x: testImages, y_: testLabels, keep_prob: 1.0})
                print("test accuracy %g" % testAcc)
                a.append(testAcc)
                # if i % 1000 == 0:
                #     testWithGalAcc = accuracy.eval(feed_dict={x: testImagesWithGal, y_: testLabelsWithGal, keep_prob: 1.0})
                #     print("test With Gal accuracy %g" % testWithGalAcc)

        print("test accuracy %g" % accuracy.eval(feed_dict={x: testImages, y_: testLabels, keep_prob: 1.0}))

        yy = y_conv.eval(feed_dict={x: testImages, y_: testLabels, keep_prob: 1.0})
        cp = correct_prediction.eval(feed_dict={x: testImages, y_: testLabels, keep_prob: 1.0})
        print(cp)
        for i in range(len(cp)):
            if cp[i] == False:
                predictedIndex = np.argmax(yy[i])
                print(i, testTypeNames[i], typeNamesList[predictedIndex])

        # SAVE THE MODEL
        saveFilename = dataDirName + "tensorflow_model.ckpt"
        saver = tf.train.Saver()
        save_path = saver.save(sess, saveFilename)
        print("Model saved in file: %s" % save_path)

        modelFilenames = [saveFilename + '.index', saveFilename + '.meta', saveFilename + '.data-00000-of-00001']

        try:
            import matplotlib.pyplot as plt
            plt.plot(a)
            plt.xlabel("Number of Epochs")
            plt.ylabel("Testing accuracy")
            plt.savefig(os.path.join(dataDirName, "testing_accuracy.png"))
            np.savetxt(os.path.join(dataDirName, "testing_accuracy.txt"), np.array(a))
        except Exception as e:
            print(e)

        try:
            calc_model_statistics(saveFilename, testImages, testLabels, testTypeNames, typeNamesList)
        except Exception as e:
            print(e)

    return modelFilenames


if __name__ == '__main__':
    t1 = time.time()
    savedFilenames = train_model('data_files/')
    t2 = time.time()
    print("time spent: {0:.2f}".format(t2 - t1))




import os
import numpy as np
import zipfile
import gzip
import pickle


def save_templates(saveFilename, trainImages, trainLabels, trainFilenames, typeNamesList):
    nLabels = len(typeNamesList)
    templateFluxesAll = []
    templateLabelsAll = []
    templateFilenamesAll = []

    for c in range(nLabels):
        templateFluxesForThisType = []
        templateLabelsForThisType = []
        templateFilenamesForThisType = []

        templateIndexes = np.where(trainLabels == c)[0]
        print(c, len(templateIndexes))
        countTemplates = 0
        for i in templateIndexes:
            templateFluxesForThisType.append(trainImages[i])
            templateLabelsForThisType.append(trainLabels[i])
            templateFilenamesForThisType.append(trainFilenames[i].replace('.lnw', '').strip('_z0.0'))
            countTemplates += 1
            if countTemplates > 100:
                break
                
        if countTemplates == 0:
            templateFluxesForThisType.append(np.zeros(len(trainImages[0])))
            templateLabelsForThisType.append(nLabels + 1)  # Out of range
            templateFilenamesForThisType.append('No Templates')
            print("No Templates %d" % c)

        print("Appending Flux %d..." % c)
        templateFluxesAll.append(np.array(templateFluxesForThisType))
        templateLabelsAll.append(np.array(templateLabelsForThisType))
        templateFilenamesAll.append(np.array(templateFilenamesForThisType))
        print("Appended %d" % c)

    # These are nLabels by numOfTemplatesForEachType dimensional arrays. Each entry in this 2D array has a 1D flux
    # They are in order of nLabels
    print("Converting to Arrays...")
    templateFluxesAll = np.array(templateFluxesAll)
    templateLabelsAll = np.array(templateLabelsAll)
    templateFilenamesAll = np.array(templateFilenamesAll)

    print("Saving...")
    np.savez_compressed(saveFilename, templateFluxesAll=templateFluxesAll, templateLabelsAll=templateLabelsAll,
                        templateFilenamesAll=templateFilenamesAll)


def create_template_set_file(dataDirName):
    scriptDirectory = os.path.dirname(os.path.abspath(__file__))
    trainingSet = dataDirName + 'training_set.zip'
    extractedFolder = dataDirName + 'training_set'
    # zipRef = zipfile.ZipFile(trainingSet, 'r')
    # zipRef.extractall(extractedFolder)
    # zipRef.close()
    os.system("unzip %s -d %s" % (trainingSet, extractedFolder))


    npyFiles = {}
    fileList = os.listdir(extractedFolder)
    for filename in fileList:
        if filename.endswith('.gz'):
            f = os.path.join(scriptDirectory, extractedFolder, filename)
            # # npyFiles[filename.strip('.npy.gz')] = gzip.GzipFile(f, 'r')
            # gzFile = gzip.open(f, "rb")
            # unCompressedFile = open(f.strip('.gz'), "wb")
            # decoded = gzFile.read()
            # unCompressedFile.write(decoded)
            # gzFile.close()
            # unCompressedFile.close()
            os.system("gzip -dk %s" % f)
            npyFiles[filename.strip('.npy.gz')] = f.strip('.gz')

    trainImages = np.load(npyFiles['trainImages'], mmap_mode='r')
    trainLabels = np.load(npyFiles['trainLabels'], mmap_mode='r')
    trainFilenames = np.load(npyFiles['trainFilenames'])
    typeNamesList = np.load(npyFiles['typeNamesList'])

    saveFilename = dataDirName + 'templates.npz'

    print("Saving Templates...")
    save_templates(saveFilename, trainImages, trainLabels, trainFilenames, typeNamesList)
    print("Saved Templates to: " + saveFilename)

    loaded = np.load(os.path.join(scriptDirectory, saveFilename))

    return saveFilename


if __name__ == '__main__':
    templateSetFilename = create_template_set_file('data_files/')



import os
import dash

directory = r'/Users/danmuth/Documents/DES_Y4_Spectra/'

# # LIST ALL SPECTRAL FILENAMES FROM DROPBOX
# filenames = []
# extensions = ('.flm', '.dat', '.txt', '.fits')
# for path, subdirs, files in os.walk(directory):
#     for name in files:
#         if name.lower().endswith(extensions):
#             filenames.append(os.path.join(path, name).replace(directory, ''))
# filenames.sort()
# for f in filenames:
#     print("('%s', 0)," % f)

atels = [
    ('Gemini/des16c1dgn-20161127-gmos.flm', 0.58),
    ('Gemini/des16c1fgm-20161230-gmos.flm', 0.363),
    ('Gemini/des16c2cva-20161109-gmos.flm', 0.404),
    # ('Gemini/des16c2dil-20161130-gmos.flm', 0.),
    ('Gemini/des16c3esw-20161229-gmos.flm', 0.67),
    ('Gemini/des16e1dcx-20161127-gmos.flm', 0.45),
    ('Gemini/des16e2cqq-20161105-gmos.flm', 0.426),
    ('Gemini/des16s1byw-20161012-gmos.flm', 0.358),
    ('Gemini/des16s2drt-20161128-gmos.flm', 0.331),
    ('Gemini/des16s2fgg-20170104-gmos.flm', 0.613),
    ('Gemini/des16x1cds-20161012-gmos.flm', 0.294),
    ('Gemini/des16x1cdu-20161013-gmos.flm', 0.289),
    ('Gemini/des16x1cpf-20161029-gmos.flm', 0.436),
    ('Gemini/des16x1der-20161128-gmos.flm', 0.453),
    ('Gemini/des16x1ewu-20161222-gmos.flm', 0.8),
    ('Gemini/des16x2cpp-20161029-gmos.flm', 0.518),
    ('Gemini/des16x2epa-20161222-gmos.flm', 0.650),
    ('Gemini/des16x3cry-20161105-gmos.flm', 0.612),
    ('Gemini/des16x3dlw-20161129-gmos.flm', 0.71),
    ('Gemini/des16x3dvb-20161223-gmos.flm', 0.329),
    # ('Keck/DES16C2nm.ms.dat', 0.),
    # ('Keck/lrisDES16C3eco.spec.txt', 0.),
    # ('Keck/lrisDES16S1ahw.spec.txt', 0.),
    ('Keck/lrisDES16S2ahx.spec.txt', 0.717),
    # ('Keck/lrisDES16S2aif.spec.txt', 0.),
    ('Keck/lrisDES16S2aif_obs2.spec.txt', 1.473),
    ('Keck/lrisDES16X1kx.spec.txt', 0.546),
    ('Keck/lrisDES16X3bji.spec.txt', 0.65),
    ('MMT/DES16S2cpe-20161102.dat', 0.384),
    ('MMT/DES16X1cph-20161102.dat', 0.49),
    ('MMT/DES16X2crr-20161102.dat', 0.50),
    ('MMT/DES16X2crt-20161102.dat', 0.567),
    # ('MMT/DES16X2csj-20161102.dat', 0.),
    # ('MMT/DES16X3crw-20161102.dat', 0.),
    ('Magellan_Chile/DES16C1cbg.txt', 0.33),
    ('Magellan_Chile/DES16C1mm.txt', 0.367),
    ('Magellan_Chile/DES16C1mm_2.txt', 0.367),
    # ('Magellan_Chile/DES16C3at.txt', 0.),
    ('Magellan_Chile/DES16E1bkp.txt', 0.541),
    ('Magellan_Chile/DES16E1bkp_2.txt', 0.541),
    ('Magellan_Chile/DES16E1bkp_3.txt', 0.541),
    ('Magellan_Chile/DES16S1bzz.txt', 0.556),
    ('Magellan_Chile/DES16S1bzz_2.txt', 0.556),
    ('Magellan_Chile/DES16S1bzz_3.txt', 0.556),
    ('Magellan_Chile/DES16S1bzz_4.txt', 0.556),
    ('Magellan_Chile/DES16S1eq.txt', 0.16),
    ('Magellan_Chile/DES16X2uj.txt', 0.322),
    ('Magellan_Chile/DES16X2uj_2.txt', 0.322),
    ('Magellan_Chile/DES16X2uj_s.txt', 0.322),
    ('Magellan_Chile/DES16X3bkz.txt', 0.07),
    ('Magellan_Chile/DES16X3bkz_2.txt', 0.07),
    ('Magellan_Chile/DES16X3gc.txt', 0.17),
    ('Magellan_RAISINS2/DES16C3boj-20161005.dat', 0.442),
    ('Magellan_RAISINS2/DES16E2bjz-20161005.dat', 0.278),
    ('Magellan_RAISINS2/DES16E2bkb-20161004.dat', 0.396), #0.41
    ('Magellan_RAISINS2/DES16S1bno-20161005.dat', 0.46),
    # ('Magellan_RAISINS2/DES16X1bky-20161004.dat', 0.),
    # ('Magellan_RAISINS2/DES16X1buy-20161005.dat', 0.),
    ('Magellan_RAISINS2/DES16X2bkq-20161005.dat', 0.334),
    # ('Magellan_UoC/DES16C1bntSpectrum.fits', 0.),
    # ('Magellan_UoC/DES16C2bnvSpectrum.fits', 0.),
    # ('Magellan_UoC/DES16C3benSpectrum.fits', 0.),
    # ('Magellan_UoC/DES16E1bssSpectrum.fits', 0.),
    # ('Magellan_UoC/DES16E1byySpectrum.fits', 0.),
    # ('Magellan_UoC/DES16E2bhtSpectrum.fits', 0.),
    # ('Magellan_UoC/DES16E2bllSpectrum.fits', 0.),
    # ('Magellan_UoC/DES16E2blzSpectrum.fits', 0.),
    # ('Magellan_UoC/DES16S1bnjSpectrum.fits', 0.),
    # ('Magellan_UoC/DES16S1bysSpectrum.fits', 0.),
    # ('VLT/DES16C1ccb_all.txt', 0.),
    ('VLT/DES16C2aiy_all.txt', 0.18),
    # ('VLT/DES16C2ayx_2016nov21_all.txt', 0.),
    # ('VLT/DES16C2ayx_2016sep24_all.txt', 0.),
    # ('VLT/DES16C2cbi_all.txt', 0.),
    # ('VLT/DES16C2dcv_all.txt', 0.),
    # ('VLT/DES16C2nm_2016nov21_all.txt', 0.),
    # ('VLT/DES16C2nm_2016oct23_all.txt', 0.),
    ('VLT/DES16C3byv_all.txt', 0.8),
    ('VLT/DES16C3bzu_all.txt', 0.62),
    ('VLT/DES16C3cv_all.txt', 0.727),
    ('VLT/DES16C3ecv_all.txt', 0.244),
    # ('VLT/DES16C3edf_all.txt', 0.),
    # ('VLT/DES16E1bss_all.txt', 0.),
    ('VLT/DES16E1ciy_all.txt', 0.173),
    ('VLT/DES16E1dcj_all.txt', 0.49),
    ('VLT/DES16E1emk_all.txt', 0.44),
    ('VLT/DES16E1oq_all.txt', 0.55),
    ('VLT/DES16E2bll_all.txt', 0.63),
    ('VLT/DES16E2cjg_all.txt', 0.48),
    ('VLT/DES16E2dci_all.txt', 0.601),
    ('VLT/DES16E2dck_all.txt', 0.34),
    # ('VLT/DES16E2of_all.txt', 0.),
    ('VLT/DES16E2yo_all.txt', 0.321),
    ('VLT/DES16E2zk_all.txt', 0.80),
    # ('VLT/DES16S1afy_all.txt', 0.),
    ('VLT/DES16S1cnk_all.txt', 0.8),
    ('VLT/DES16S2bdu_all.txt', 0.69),
    # ('VLT/DES16S2bpf_all.txt', 0.),
    ('VLT/DES16S2dgb_all.txt', 0.57),
    ('VLT/DES16S2ejs_all.txt', 0.38),
    # ('VLT/DES16X1ept_all.txt', 0.),
    ('VLT/DES16X2aey_all.txt', 0.210),
    # ('VLT/DES16X2afc_all.txt', 0.),
    ('VLT/DES16X3bdj_all.txt', 0.200),
    ('VLT/DES16X3bfv_all.txt', 0.60),
    ('VLT/DES16X3cdv_all.txt', 0.824),
    ('VLT/DES16X3cer_all.txt', 1.16),
    ('VLT/DES16X3cxy_all.txt', 0.688), #0.50
    # ('VLT/DES16X3eyo_all.txt', 0.)
    ]


filenames = [os.path.join(directory, i[0]) for i in atels]
knownRedshifts = [i[1] for i in atels]

classification = dash.Classify(filenames, knownRedshifts, classifyHost=False)
bestFits, redshifts, bestTypes, rejectionLabels, reliableFlags = classification.list_best_matches(n=5)

# SAVE BEST MATCHES
print(bestFits)
f = open('classification_results.txt', 'w')
for i in range(len(atels)):
    f.write("%s   z=%s     %s     %s      %s\n %s\n\n" % (atels[i][0], atels[i][1], bestTypes[i], reliableFlags[i], rejectionLabels[i], bestFits[i]))
f.close()
print("Finished classifying %d spectra!" % len(atels))

# PLOT SPECTRUM ON GUI
classification.plot_with_gui(indexToPlot=1)




from dash.training_parameters import create_training_params_file
from dash.create_training_set import create_training_set_files
from dash.create_template_set import create_template_set_file
from dash.deep_learning_multilayer import train_model
import zipfile
import os
import shutil
import time

if __name__ == '__main__':
    dataDirName = 'data_files_test/'
    dataFilenames = []
    if not os.path.exists(dataDirName):
        os.makedirs(dataDirName)

    # MAKE INFO TEXT FILE ABOUT MODEL
    modelInfoFilename = dataDirName + "model_info.txt"
    with open(modelInfoFilename, "w") as f:
        f.write("Date Time: %s\n" % time.strftime("%Y-%m-%d %H:%M:%S", time.gmtime()))
        f.write("Directory: %s\n" % dataDirName)
        f.write("Add Host: True\n")
        f.write("SN-Host fractions: [0.01, 0.02, 0.05, 0.07, 0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9]\n")
        f.write("Classify Host: True\n")
        f.write("Redshift: Zero\n")
        f.write("Redshift Range: 0 to 0.\n")
        f.write("Redshift Precision: 0.02\n")
        f.write("Fraction of Training Set Used: 0.9")
        f.write("Training Amount: 50 x 400000\n")
        dataFilenames.append(modelInfoFilename)

    # CREATE PARAMETERS PICKLE FILE
    t1 = time.time()
    trainingParamsFilename = create_training_params_file(dataDirName)
    dataFilenames.append(trainingParamsFilename)
    t2 = time.time()
    print("time spent: {0:.2f}".format(t2 - t1))

    # CREATE TRAINING SET FILES
    trainingSetFilename = create_training_set_files(dataDirName, minZ=0, maxZ=0., redshiftPrecision=0.01, trainWithHost=False, classifyHost=False)
    dataFilenames.append(trainingSetFilename)
    t3 = time.time()
    print("time spent: {0:.2f}".format(t3 - t2))

    # TRAIN TENSORFLOW MODEL
    modelFilenames = train_model(dataDirName)
    dataFilenames.extend(modelFilenames)
    t4 = time.time()
    print("time spent: {0:.2f}".format(t4 - t3))

    # SAVE ALL FILES TO ZIP FILE
    dataFilesZip = 'data_files_zeroZ_classifyHost_v01.zip'
    with zipfile.ZipFile(dataFilesZip, 'w') as myzip:
        for f in dataFilenames:
            myzip.write(f)

    modelZip = 'model_zeroZ_classifyHost_v01.zip'
    with zipfile.ZipFile(modelZip, 'w') as myzip:
        for f in [dataFilenames[0]] + dataFilenames[2:]:
            myzip.write(f)

    # Delete data_files folder since they are now in the zip files
    # for filename in filenames:
    #     os.remove(filename)



import sys
from dash.download_data_files import download_all_files

download_all_files('v01')

try:
    from PyQt5 import QtGui
    from dash.gui_main import MainApp
except ImportError:
    print("Warning: You will need to install 'PyQt5' if you want to use the graphical interface. " \
          "Using the automatic library will continue to work.")

from dash.classify import Classify


def main():
    classification = Classify(filenames=['test_spectrum_file.dat',
                                         'test_spectrum_file.dat'],
                              redshifts=[0.34, 0.13])
    print(classification.list_best_matches())
    # classification.plot_with_gui(indexToPlot=1)


def run_gui():
    app = QtGui.QApplication(sys.argv)
    form = MainApp()
    form.show()
    app.exec_()


if __name__ == '__main__':
    main()
    # run_gui()





import numpy as np


def labels_indexes_to_arrays(labelsIndexes, nLabels):
    numArrays = len(labelsIndexes)
    labelsIndexes = labelsIndexes
    labels = np.zeros((numArrays, nLabels))
    labels[np.arange(numArrays), labelsIndexes] = 1

    return labels


def zero_non_overlap_part(array, minIndex, maxIndex):
    slicedArray = np.copy(array)
    slicedArray[0:minIndex] = np.zeros(minIndex)
    slicedArray[maxIndex:] = np.zeros(len(array)-maxIndex)

    return slicedArray


def normalise_spectrum(flux):
    fluxNorm = (flux - min(flux)) / (max(flux) - min(flux))

    return fluxNorm


import pickle


def create_training_params_file(dataDirName):
    parameters = {
        'typeList': ['Ia-norm', 'Ia-91T', 'Ia-91bg', 'Ia-csm', 'Ia-02cx', 'Ia-pec',
                     'Ib-norm', 'Ibn', 'IIb', 'Ib-pec', 'Ic-norm', 'Ic-broad',
                     'Ic-pec', 'IIP', 'IIL', 'IIn', 'II-pec'],
        'nTypes': 17,
        'w0': 2500.,  # wavelength range in Angstroms
        'w1': 10000.,
        'nw': 1024,  # number of wavelength bins
        'minAge': -20.,
        'maxAge': 50.,
        'ageBinSize': 4.,
        'galTypeList': ['E', 'S0', 'Sa', 'Sb', 'Sc', 'SB1', 'SB2', 'SB3', 'SB4', 'SB5', 'SB6']
    }

    trainingParamsFilename = dataDirName + 'training_params.pickle'

    # Saving the objects:
    with open(trainingParamsFilename, 'wb') as f:
        pickle.dump(parameters, f, protocol=2)
    print("Saved files to %s" % trainingParamsFilename)

    # Getting back the objects:
    with open(trainingParamsFilename, 'rb') as f:
        pars = pickle.load(f)
        nTypes, w0, w1, nw, minAge, maxAge, ageBinSize, typeList, galTypeList = pars['nTypes'], pars['w0'], pars['w1'], \
                                                                                pars['nw'], pars['minAge'], \
                                                                                pars['maxAge'], pars['ageBinSize'], \
                                                                                pars['typeList'], pars['galTypeList']

    return trainingParamsFilename


if __name__ == '__main__':
    trainingParamsFilename1 = create_training_params_file('data_files/')



from dash.preprocessing import ReadSpectrumFile
from dash.create_arrays import TempList
import pickle
import os
import gzip


class SaveTemplateSpectra(object):
    def __init__(self, parameterFile):
        with open(parameterFile, 'rb') as f:
            pars = pickle.load(f)
            self.w0, self.w1, self.nw = pars['w0'], pars['w1'], pars['nw']

    def read_template_file(self, filename):
        readSpectrumFile = ReadSpectrumFile(filename, self.w0, self.w1, self.nw)
        spectrum = readSpectrumFile.file_extension()

        return spectrum

    def template_spectra_to_list(self, tempFileList, templateDirectory):
        tempList = TempList().temp_list(tempFileList)
        templates = []
        for filename in tempList:
            spectrum = self.read_template_file(templateDirectory+filename)
            templates.append(spectrum)
            print(filename)

        return templates

    def save_templates(self, snTempFileList, snTemplateDirectory, galTempFileList, galTemplateDirectory, saveFilename):
        snTemplates = self.template_spectra_to_list(snTempFileList, snTemplateDirectory)
        galTemplates = self.template_spectra_to_list(galTempFileList, galTemplateDirectory)
        templates = {'sn': snTemplates, 'gal': galTemplates}

        # Saving the objects
        with gzip.open(saveFilename, 'wb') as f:
            pickle.dump(templates, f, protocol=2)
        print("Saved templates to %s" % saveFilename)


def save_templates():
    scriptDirectory = os.path.dirname(os.path.abspath(__file__))

    snidTemplateDirectory = os.path.join(scriptDirectory, "../templates/snid_templates_Modjaz_BSNIP/")
    snidTempFileList = snidTemplateDirectory + 'templist.txt'
    galTemplateDirectory = os.path.join(scriptDirectory, "../templates/superfit_templates/gal/")
    galTempFileList = galTemplateDirectory + 'gal.list'

    saveTemplateSpectra = SaveTemplateSpectra('data_files/training_params.pickle')
    saveFilename = 'data_files/sn_and_gal_templates.pklz'
    saveTemplateSpectra.save_templates(snidTempFileList, snidTemplateDirectory, galTempFileList, galTemplateDirectory, saveFilename)

    return saveFilename


if __name__ == '__main__':
    templatesFilename1 = save_templates()



from selenium import webdriver
import time
import pandas as pd

USERNAME = 'danmuth'
PASSWORD = 'CKdjlor4N!'


def login_to_atc():
    browser = webdriver.Chrome()
    browser.get('https://portal-auth.nersc.gov/atc2/web/targets/DES16X2gap')
    time.sleep(10)
    username = browser.find_element_by_id("j_username")
    password = browser.find_element_by_id("j_password")
    username.send_keys(USERNAME)
    password.send_keys(PASSWORD)
    login_attempt = browser.find_element_by_xpath("//*[@type='submit']")
    login_attempt.submit()
    time.sleep(10)
    return browser


def open_page_and_download_spectrum(browser, objectName):
    browser.get('https://portal-auth.nersc.gov/atc2/web/spectra/' + objectName)
    time.sleep(10)
    downloaded = False
    files = []
    elems = browser.find_elements_by_xpath("//a[@href]")
    for elem in elems:
        if '/spectrum/' in elem.get_attribute("href"):
            fileToDownload = elem.get_attribute("href")
            files.append(fileToDownload)
            linkText = elem.get_attribute("innerText")
            if 'aat' not in linkText:
                print(linkText, fileToDownload)
                browser.get(fileToDownload)
                downloaded = True
    if not downloaded and files != []:
        browser.get(files[0])


def get_object_names(filename):
    df = pd.read_csv(filename, sep=',', header=1)
    ozdesList = df.values
    names = ozdesList[:, 1]
    numTransients = len(names)
    return names


if __name__ == '__main__':
    names = get_object_names('91bg_candidates.csv')
    browser1 = login_to_atc()
    for name in names:
        open_page_and_download_spectrum(browser1, name)
    # browser1.close()



import zipfile
import os


def unzip_data_files(dataZipFilename):
    print("Unzipping data files...")
    scriptDirectory = os.path.dirname(os.path.abspath(__file__))
    zipRef = zipfile.ZipFile(os.path.join(scriptDirectory, dataZipFilename), 'r')
    zipRef.extractall(os.path.join(scriptDirectory, dataZipFilename.strip(".zip")))
    zipRef.close()
    os.remove(os.path.join(scriptDirectory, dataZipFilename))
    print("Data files installed!")


if __name__ == '__main__':
    unzip_data_files('models_v01.zip')


from scipy.fftpack import fft
from scipy.signal import argrelmax
import numpy as np
from scipy.stats import chisquare, pearsonr
# import matplotlib
# matplotlib.use('TkAgg')
# import matplotlib.pyplot as plt


def combined_prob(bestMatchList):
    host, prevName, age, prob = bestMatchList[0]
    probInitial = float(prob)
    bestName = prevName
    prevBroadTypeName = prevName[0:2]
    prevMinAge, prevMaxAge = age.split(' to ')
    probTotal = 0.
    agesList = [int(prevMinAge), int(prevMaxAge)]
    probPossible = 0.
    agesListPossible = []
    probPossible2 = 0.
    agesListPossible2 = []
    index = 0
    for host, name, age, prob in bestMatchList[0:10]:
        index += 1
        minAge, maxAge = list(map(int, age.split(' to ')))
        if "IIb" in name:
            broadTypeName = "Ib"
        else:
            broadTypeName = name[0:2]
        if name == prevName:
            if probPossible == 0:
                if (minAge in agesList) or (maxAge in agesList):
                    probTotal += float(prob)
                    prevName = name
                    agesList = agesList + [minAge, maxAge]

                else:
                    probPossible = float(prob)
                    agesListPossible = [minAge, maxAge]
            else:
                if probPossible2 == 0:
                    if ((minAge in agesListPossible) or (maxAge in agesListPossible)) and (
                                (minAge in agesList) or (maxAge in agesList)):
                        probTotal += probPossible + float(prob)
                        agesList = agesList + agesListPossible + [minAge, maxAge]
                        probPossible = 0
                        agesListPossible = []
                    else:
                        probPossible2 = probPossible + float(prob)
                        agesListPossible2 = agesListPossible + [minAge, maxAge]
                else:
                    if ((minAge in agesListPossible2) or (maxAge in agesListPossible2)) and (
                                (minAge in (agesList + agesListPossible)) or (maxAge in (agesList + agesListPossible))):
                        probTotal += probPossible2 + float(prob)
                        agesList = agesList + agesListPossible2 + [minAge, maxAge]
                        probPossible, probPossible2 = 0, 0
                        agesListPossible, agesListPossible2 = [], []

        elif broadTypeName == prevBroadTypeName:
            if probPossible == 0:
                if (minAge in agesList) or (maxAge in agesList):
                    if index <= 2:
                        probTotal += float(prob)
                        agesList = agesList + [minAge, maxAge]
                        bestName = broadTypeName
                    elif bestName == broadTypeName:
                        probTotal += float(prob)
                        agesList = agesList + [minAge, maxAge]
                    else:
                        print("Something strange going on...")
                else:
                    probPossible = float(prob)
                    agesListPossible = [minAge, maxAge]
            else:
                if ((minAge in agesListPossible) or (maxAge in agesListPossible)) and (
                            (minAge in agesList) or (maxAge in agesList)):
                    probTotal += probPossible + float(prob)
                    agesList = agesList + agesListPossible + [minAge, maxAge]
                    probPossible = 0
                    agesListPossible = []
                else:
                    break
        else:
            break

    bestAge = '%d to %d' % (min(agesList), max(agesList))

    if probTotal > probInitial:
        reliableFlag = True
    else:
        reliableFlag = False

    return host, bestName, bestAge, round(probTotal, 4), reliableFlag


class FalsePositiveRejection(object):
    def __init__(self, inputFlux, templateFluxes):
        self.inputFlux = inputFlux
        self.templateFluxes = templateFluxes
        self.nw = len(self.inputFlux)

    def _cross_correlation(self, templateFlux):
        inputfourier = fft(self.inputFlux, self.nw)
        tempfourier = fft(templateFlux, self.nw)

        product = inputfourier * np.conj(tempfourier)
        xcorr = fft(product)

        rmsinput = np.std(inputfourier)
        rmstemp = np.std(tempfourier)

        xcorrnorm = (1. / (self.nw * rmsinput * rmstemp)) * xcorr

        rmsxcorr = np.std(product)

        xcorrnormRearranged = np.concatenate((xcorrnorm[int(len(xcorrnorm) / 2):], xcorrnorm[0:int(len(xcorrnorm) / 2)]))

        #
        # w0 = 2500.  # wavelength range in Angstroms
        # w1 = 11000.
        # nw = 1024.  # number of wavelength bins
        # dwlog = np.log(w1 / w0) / self.nw
        # zaxisindex1 = np.concatenate((np.arange(-self.nw / 2, 0), np.arange(0, self.nw / 2)))
        # zaxis1 = np.zeros(self.nw)
        # zaxis1[0:(self.nw/2 -1)] = (np.exp(abs(zaxisindex1[0:(self.nw/2-1)]) * dwlog) - 1)
        # zaxis1[self.nw/2:] = -(np.exp(abs(zaxisindex1[(self.nw/2):]) * dwlog) - 1)
        # plt.plot(zaxis1, xcorrnormRearranged)
        # plt.show()

        return xcorr, rmsinput, rmstemp, xcorrnorm, rmsxcorr, xcorrnormRearranged

    def _get_peaks(self, crosscorr):
        peakindexes = argrelmax(crosscorr)[0]

        ypeaks = []
        for i in peakindexes:
            ypeaks.append(abs(crosscorr[i]))

        arr = list(zip(*[peakindexes, ypeaks]))
        arr.sort(key=lambda x: x[1])
        sortedPeaks = list(reversed(arr))

        return sortedPeaks


    def _calculate_r(self, crosscorr):
        deltapeak1, h1 = self._get_peaks(crosscorr)[0]  # deltapeak = np.argmax(abs(crosscorr))
        deltapeak2, h2 = self._get_peaks(crosscorr)[1]
        # h = crosscorr[deltapeak]
        rmsxcorr = np.std(crosscorr)
        ##    shift = deltapeak
        ##    arms = 0
        ##    srms = 0
        ##    for k in range(0,int(nw-1)):
        ##        angle = -2*np.pi*k/nw
        ##        phase = complex(np.cos(angle),np.sin(angle))
        ####	f=2
        ####	if (k < k2):
        ####		arg = np.pi * float(k-k1)/float(k2-k1)
        ####		f = f * .25 * (1-np.cos(arg)) * (1-np.cos(arg))
        ####	elif (k > k3):
        ####		arg = np.pi * float(k-k3)/float(k4-k3)
        ####		f = f * .25 * (1+np.cos(arg)) * (1+np.cos(arg))
        ##        arms = arms + (phase*crosscorr[k+1]).imag*(phase*crosscorr[k+1]).imag
        ##        srms = srms + (phase*crosscorr[k+1]).real*(phase*crosscorr[k+1]).real
        ##
        ##    arms = np.sqrt(arms)/nw
        r = abs(h1 / (np.sqrt(2) * rmsxcorr))
        fom = (h1 - 0.05) ** 0.75 * (h1 / h2)

        return r, deltapeak1, fom

    def calculate_rlap(self, crosscorr, templateFlux):
        r, deltapeak, fom = self._calculate_r(crosscorr)
        shift = deltapeak - self.nw / 2  # shift from redshift

        # lap value
        iminindex, imaxindex = self.min_max_index(self.inputFlux)
        tminindex, tmaxindex = self.min_max_index(templateFlux)

        overlapminindex = max(iminindex + shift, tminindex)
        overlapmaxindex = min(imaxindex - 1 + shift, tmaxindex - 1)

        lap = np.log(overlapmaxindex / overlapminindex)
        rlap = r * lap


        fom = fom * lap
        # print r, lap, rlap, fom
        return r, lap, rlap, fom

    def min_max_index(self, flux):
        minindex, maxindex = (0, self.nw - 1)
        zeros = np.where(flux == 0)[0]
        j = 0
        for i in zeros:
            if (i != j):
                break
            j += 1
            minindex = j
        j = int(self.nw) - 1
        for i in zeros[::-1]:
            if (i != j):
                break
            j -= 1
            maxindex = j

        return minindex, maxindex

    def calculate_chi_squared(self, templateFlux):
        """ Only calculate on overlap region (MAYBE AVERAGE ALL TEMPLATES LATER)"""
        iminindex, imaxindex = self.min_max_index(self.inputFlux)
        tminindex, tmaxindex = self.min_max_index(templateFlux)

        overlapminindex = max(iminindex, tminindex)
        overlapmaxindex = min(imaxindex - 1, tmaxindex - 1)

        inputSpecOverlapped = 100 * (1+self.inputFlux[overlapminindex:overlapmaxindex])
        templateSpecOverlapped = 100 * (1+templateFlux[overlapminindex:overlapmaxindex])


        chi2 = chisquare(inputSpecOverlapped, templateSpecOverlapped)[0]
        pearsonCorr = pearsonr(inputSpecOverlapped, templateSpecOverlapped)

        return chi2, pearsonCorr

    def rejection_label(self):
        chi2List = []
        pearsonList = []
        for templateFlux in self.templateFluxes:
            chi2 = self.calculate_chi_squared(templateFlux)[0]
            chi2List.append(chi2)
            pearson = self.calculate_chi_squared(templateFlux)[1]
            pearsonList.append(pearson)

        chi2Mean = round(np.mean(chi2List),2)
        pearsonMean = np.mean(pearsonList)
        print(chi2Mean, np.median(chi2List), min(chi2List), max(chi2List), len(chi2List))
        print(pearsonMean, np.median(pearsonList), min(pearsonList), max(pearsonList), len(pearsonList))

        return "%s, Pearson=%s" % (str(chi2Mean), str(pearsonMean))

    def rejection_label2(self):
        rlapList = []
        for templateFlux in self.templateFluxes:
            xcorr, rmsinput, rmstemp, xcorrnorm, rmsxcorr, xcorrnormRearranged = self._cross_correlation(templateFlux.astype('float'))
            crosscorr = xcorrnormRearranged
            r, lap, rlap, fom = self.calculate_rlap(crosscorr, templateFlux)
            rlapList.append(rlap)

        rlapMean = round(np.mean(rlapList),2)
        print(rlapMean, np.median(rlapList), min(rlapList), max(rlapList))

        return str(rlapMean)




import os
import dash
import pandas as pd
import numpy as np
import scipy
import shutil

dfNewList = pd.read_csv('../templates/typeIa_withcuts_Moller090517.csv', sep=',', header=1)
newList = dfNewList.values
cidNew = newList[:, 2]
zNew = newList[:, 43] #[-1]
numTransientsNewList = len(cidNew)

df = pd.read_csv('../templates/DES3Y_SNIa.csv', sep=',', header=1)
ozdesList = df.values
numTransients = len(ozdesList[:,0])
names = ozdesList[:, 1]
fields = ozdesList[:, 2]
snid = ozdesList[:, 0]
for i in range(numTransients):
    ozdesList[i,2] = str(ozdesList[i,2]).replace(",", '&')
redshifts = ozdesList[:,3]
specType = ozdesList[:,4]
ozdesList = np.hstack((ozdesList, np.zeros((numTransients,1))))
print(ozdesList)

for cid in cidNew:
    if cid not in snid:
        print(cid)
        print("CID NOT IN LIST\n\n")

directory1 = '../templates/maipenrai/'  # r'/priv/maipenrai/ozdes/ftp/data/'
directory2 = '../templates/non_aat/'

# LIST ALL SPECTRAL FILENAMES
filenames = []
extensions = ('.dat', '.fits', '.txt')
# This loops goes first so that the files in 'copiedFiles' form maipenrai will replace the downloaded ATC spectra
for path, subdirs, files in os.walk(directory1):
    for name in files:
        if name.lower().endswith(extensions):
            filenames.append(os.path.join(path, name))

for path, subdirs, files in os.walk(directory2):
    for name in files:
        if name.lower().endswith(extensions):
            filenames.append(os.path.join(path, name))

# for f in filenames:
#     print("('%s', 0)," % f)


for i in range(numTransients):
    for f in filenames:
        if names[i] in f:
            ozdesList[i,5] = f # If there is more than one entry, then it will take the last one in the filenames list

print(ozdesList)

names = []
filenames = []
knownRedshifts = []
for (snid, name, field, z, specType, filename) in ozdesList:
    if filename != 0:
        filenames.append(filename)
        if snid in cidNew:
            newListIndex = np.where(cidNew == snid)[0][0]
            # print(z-zNew[newListIndex])
            z = zNew[newListIndex]
        knownRedshifts.append(z)
        names.append(name)
        # shutil.copy2(directory+filename, './copiedFiles')
import time
t1 = time.time()
classification = dash.Classify(filenames, knownRedshifts, classifyHost=False, smooth=7, knownZ=True)
bestFits, redshifts, bestTypes, rejectionLabels, reliableFlags = classification.list_best_matches(n=5)
t2 = time.time()
print(t2-t1)
# SAVE BEST MATCHES
print(bestFits)
f = open('classification_results.txt', 'w')
for i in range(len(filenames)):
    f.write("%s   z=%s     %s      %s     %s\n %s\n\n" % (names[i], redshifts[i], bestTypes[i], reliableFlags[i], rejectionLabels[i], bestFits[i]))
f.close()
print("Finished classifying %d spectra!" % len(filenames))

ozdesList = np.hstack((ozdesList, np.zeros((numTransients,11))))
for i in range(numTransients):
    for j in range(len(knownRedshifts)):
        if names[j] in str(ozdesList[i,5]):
            ozdesList[i,7:10] = bestTypes[j][0:3]
            ozdesList[i,10] = str(reliableFlags[j])
            ozdesList[i,11] = ' | '.join(bestFits[j][0])
            ozdesList[i,12] = ' | '.join(bestFits[j][1])
            ozdesList[i,13] = ' | '.join(bestFits[j][2])
            ozdesList[i,14] = ' | '.join(bestFits[j][3])
            ozdesList[i,15] = ' | '.join(bestFits[j][4])
            ozdesList[i, 16] = rejectionLabels[j]  # round(float(rejectionLabels[j]),3)
            snid = ozdesList[i, 0]
            if snid in cidNew:
                newListIndex = np.where(cidNew == snid)[0][0]
                ozdesList[i, 6] = str(zNew[newListIndex])
            else:
                ozdesList[i, 6] = '-'
                # print(ozdesList[i, 1], snid)

print(ozdesList)
# ozdesList = np.delete(ozdesList, [5], axis=1)
#ozdesList1 = ozdesList[:,0:11]
ozdesList2 = ozdesList
print(ozdesList)
print(ozdesList[1])
#ozdesList1 = np.insert(ozdesList1, 0, np.array([['SNID', 'TRANSIENT_NAME', 'FIELD', 'Z_SPEC', 'SPEC_TYPE', 'FILENAME', 'DASH_TYPE', 'DASH_AGE', 'DASH_PROB', 'RELIABLE?', 'RLAP']]), axis=0)
ozdesList2 = np.insert(ozdesList2, 0, np.array([['SNID', 'TRANSIENT_NAME', 'FIELD', 'Z_SPEC', 'SPEC_TYPE', 'FILENAME', 'Z_NEW_LIST', 'DASH_TYPE', 'DASH_AGE', 'DASH_PROB', 'RELIABLE?', 'RANK_1', 'RANK_2', 'RANK_3', 'RANK_4', 'RANK_5', 'REJECTION_SCORES']]), axis=0)

#np.savetxt('DES3Y_SNIa_include_DASH_without_ranking.csv', ozdesList1, delimiter=",", fmt='%s')
np.savetxt('../templates/DES3Y_SNIa_include_DASH_withHost_withNoise.csv', ozdesList2, delimiter=",", fmt='%s')

classification.plot_with_gui(indexToPlot=20)
